<?php
/**
 * Contain admin helper functions
 *
 * @class       AW_Admin_Helper
 * @package     AutomateWoo/Admin
 * @since       2.0.3
 */

class AW_Admin_Helper
{

	/**
	 * Base JS
	 */
	public function enqueue_admin_js()
	{
		// no autosave
		wp_dequeue_script('autosave');

		wp_register_script(
			'jquery-cookie', WC()->plugin_url() . '/assets/js/jquery-cookie/jquery.cookie.js',
			array( 'jquery' ), '1.4.1'
		);

		wp_enqueue_script( 'woocommerce_admin' );
		wp_enqueue_script( 'wc-enhanced-select' );
		wp_enqueue_script( 'jquery-ui-sortable' );
		wp_enqueue_script( 'jquery-tiptip' );
		wp_enqueue_script( 'jquery-cookie' );
		wp_enqueue_script( 'jquery-ui-autocomplete' );
	}




	/**
	 * @param $view
	 * @param array $args
	 * @param mixed $path
	 */
	public function get_view( $view, $args = array(), $path = false )
	{
		if ( $args && is_array( $args ) )
			extract( $args );

		if ( ! $path )
			$path = AutomateWoo()->plugin_path . 'admin/views/';

		include( $path . $view . '.php' );
	}



	/**
	 * @param $id
	 * @param $title
	 * @param $callback
	 * @param null $screen
	 * @param string $context
	 * @param string $priority
	 * @param null $callback_args
	 */
	public function add_meta_box( $id, $title, $callback, $screen = null, $context = 'advanced', $priority = 'default', $callback_args = null )
	{
		$id = AutomateWoo()->plugin_prefix . $id;

		add_meta_box( $id, $title, $callback, $screen, $context, $priority, $callback_args );

		add_filter("postbox_classes_{$screen}_{$id}", array( $this, 'inject_postbox_class' ) );
	}


	/**
	 * @param $classes
	 *
	 * @return array
	 */
	function inject_postbox_class( $classes )
	{
		$classes[] = 'aw-metabox';
		return $classes;
	}


	/**
	 * @param $type (warning,error,success)
	 * @param $strong
	 * @param string $more
	 * @param string $class
	 * @param string $button_text
	 * @param string $button_link
	 * @param string $button_class
	 */
	function notice( $type, $strong, $more = '', $class = '', $button_text = '', $button_link = '', $button_class = '' )
	{
		?>
		<div class="notice notice-<?php echo $type ?> automatewoo-notice <?php echo $class ?>">
			<p>
				<strong><?php echo $strong; ?></strong> <?php echo $more; ?>
			</p>
			<?php if ($button_text): ?>
				<p><a href="<?php echo $button_link; ?>" class="button-primary <?php echo $button_class; ?>"><?php echo $button_text; ?></a></p>
			<?php endif; ?>
		</div>
	<?php
	}

}



