<?php

/**
 * Class AW_Admin
 */
class AW_Admin
{

	/**
	 * @var array
	 */
	protected $js_params = array();



	/**
	 * Add filters and action hooks
	 */
	public function __construct()
	{
		add_action('admin_head', array( $this, 'admin_head' ) );
		add_action('admin_head', array( $this, 'enqueue_scripts' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'register_scripts' ) );
		add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles' ) );
		add_action('save_post', array( $this, 'save_post' ) );
		add_action('pre_get_posts', array( $this, 'set_default_workflow_order') );

		if ( apply_filters( 'automatewoo_show_ui', true, 'settings' ) )
			add_filter( 'woocommerce_get_settings_pages', array( $this, 'add_settings_page' ) );

		if ( apply_filters( 'automatewoo_show_ui', true, 'reports' ) )
			add_filter( 'woocommerce_admin_reports', array( $this, 'register_reports' ) );

		add_filter( 'manage_posts_columns' , array( $this, 'add_columns' ) );
		add_filter( 'manage_posts_custom_column' , array( $this, 'add_column_data' ), 10 , 2 );
		add_action( 'admin_notices', array( $this, 'licence_notices') );
	}


	/**
	 *
	 */
	public function register_scripts()
	{
		$url = AutomateWoo()->plugin_url . 'admin/assets/js/min/';

		wp_register_script( 'automatewoo', $url . 'automatewoo.min.js', array('jquery'), AutomateWoo()->version );
		wp_register_script( 'automatewoo-workflows', $url . 'automatewoo-workflows.min.js', array('automatewoo'), AutomateWoo()->version );
		wp_register_script( 'automatewoo-tools', $url . 'automatewoo-tools.min.js', array('automatewoo'), AutomateWoo()->version );
		wp_register_script( 'automatewoo-sms-test', $url . 'automatewoo-sms-test.min.js', array('automatewoo'), AutomateWoo()->version );
		wp_register_script( 'automatewoo-email-preview', $url . 'automatewoo-email-preview.min.js', array('automatewoo'), AutomateWoo()->version );
		wp_register_script( 'automatewoo-activate', $url . 'automatewoo-activate.min.js', array('automatewoo'), AutomateWoo()->version );
		wp_register_script( 'automatewoo-modal', $url . 'automatewoo-modal.min.js', array('automatewoo'), AutomateWoo()->version );
	}


	/**
	 * Load styles earlier than scripts to avoid flash of un-styled workflows UI
	 */
	function enqueue_styles()
	{
		wp_register_style(
			'automatewoo-main',
			AutomateWoo()->plugin_url . 'admin/assets/css/aw-main.css',
			array(),
			AutomateWoo()->version
		);

		wp_enqueue_style( 'automatewoo_admin_styles', WC()->plugin_url() . '/assets/css/admin.css', array(), WC_VERSION );
		wp_enqueue_style('automatewoo-main');

		// load main js object
		wp_enqueue_script( 'automatewoo' );
	}



	/**
	 * Enqueue scripts
	 * Do this on the admin_head action so we have access to the post object
	 */
	function enqueue_scripts()
	{
		global $post;

		// Workflow editing page
		if ( $post && $post->post_type == AutomateWoo()->post_type )
		{
			AutomateWoo()->admin_helper->enqueue_admin_js();

			wp_localize_script(
				'automatewoo-workflows',
				'aw_params',
				$this->get_js_params()
			);

			wp_enqueue_script('automatewoo-workflows');
		}
	}



	/**
	 *  admin_head
	 */
	function admin_head()
	{
		global $post;

		if ( ! $post )
			return;

		if ( $post->post_type != AutomateWoo()->post_type )
			return;

		$this->add_js_param( 'workflow_id', $post->ID );
		$this->add_js_param( 'is_new_workflow', $post->post_status == 'auto-draft' );


		// Pass action data map
		$actions_data = array();

		foreach ( AutomateWoo()->get_actions() as $action )
		{
			$actions_data[ $action->get_name() ] = array(
				'can_be_previewed' => $action->can_be_previewed,
				'required_data_items' => $action->required_data_items
			);
		}

		$this->add_js_param('actions_data', $actions_data);


		// Load global $workflow object
		if ( $post->post_status !== 'auto-draft' )
		{
			$GLOBALS['workflow'] = new AW_Model_Workflow( $post );
		}

		AutomateWoo()->admin_helper->add_meta_box( 'trigger_box',
			__('Trigger','automatewoo'), array($this, 'meta_box_triggers'),
			AutomateWoo()->post_type, 'normal', 'high'
		);

		AutomateWoo()->admin_helper->add_meta_box( 'actions_box',
			__('Actions','automatewoo'), array($this, 'meta_box_actions'),
			AutomateWoo()->post_type, 'normal', 'high'
		);

		AutomateWoo()->admin_helper->add_meta_box( 'options_box',
			__('Options','automatewoo'), array($this, 'meta_box_options'),
			AutomateWoo()->post_type, 'side'
		);

		AutomateWoo()->admin_helper->add_meta_box( 'text_vars_box',
			__('Variables','automatewoo'), array($this, 'meta_box_text_vars'),
			AutomateWoo()->post_type, 'side'
		);

	}



	/**
	 *
	 */
	public function meta_box_triggers()
	{
		global $workflow;

		if ( $workflow )
		{
			$selected_trigger = $workflow->get_trigger();
		}
		else
		{
			$selected_trigger = false;
		}


		AutomateWoo()->admin_helper->get_view('meta-box-trigger', array(
			'workflow' => $workflow,
			'selected_trigger' => $selected_trigger
		));
	}


	/**
	 *
	 */
	public function meta_box_actions()
	{
		global $workflow;

		if ( $workflow )
			$actions = $workflow->get_actions();
		else
			$actions = false;

		$action_select_box_values = array();

		foreach ( AutomateWoo()->get_actions() as $registered_action )
		{
			$action_select_box_values[$registered_action->group][$registered_action->get_name()] = $registered_action->get_title();
		}


		AutomateWoo()->admin_helper->get_view('meta-box-actions', array(
			'workflow' => $workflow,
			'actions' => $actions,
			'action_select_box_values' => $action_select_box_values
		));
	}


	/**
	 *
	 */
	public function meta_box_text_vars()
	{
		AutomateWoo()->admin_helper->get_view('meta-box-text-vars');
	}


	/**
	 *
	 */
	public function meta_box_options()
	{
		global $workflow;

		AutomateWoo()->admin_helper->get_view('meta-box-options', array(
			'workflow' => $workflow,
		));
	}


	/**
	 * @param $name
	 * @param $value
	 */
	function add_js_param( $name, $value )
	{
		$this->js_params[$name] = $value;
	}


	/**
	 * @return array
	 */
	function get_js_params()
	{
		return $this->js_params;
	}


	/**
	 * @param $post_id
	 */
	function save_post( $post_id )
	{
		if ( aw_request('post_type') != AutomateWoo()->post_type )
			return;

		// flush caches
		AutomateWoo()->cache->delete('abandoned_cart_in_use');


		$data = aw_request('aw_workflow_data');


		if ( ! is_array( $data ) )
			return;

		// Some validation for the trigger
		if ( isset( $data['trigger_name'] ) )
		{
			$trigger_name = $data['trigger_name'];

			if ( $trigger = AutomateWoo()->get_registered_trigger( $trigger_name ) )
			{

				// If queueing is disabled for the trigger force when to run option
				if ( ! $trigger->allow_queueing )
				{
					$data['workflow_options']['when_to_run'] = 'immediately';
				}
			}
		}


		// Save the data into meta
		foreach ( $data as $key => $value )
		{
			update_post_meta( $post_id, $key, $value );
		}
	}



	/**
	 * @param $reports
	 *
	 * @return mixed
	 */
	function register_reports($reports)
	{
		$reports['automatewoo'] = array(
			'title'  => __( 'AutomateWoo', 'woocommerce' ),
			'reports' => array(
				'runs_by_date' => array(
					'title'       => __( 'Run History', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				"email_tracking" => array(
					'title' => __( 'Email Tracking', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				"conversions" => array(
					'title' => __( 'Conversions', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				"conversions_list" => array(
					'title' => __( 'Conversions List', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				"carts" => array(
					'title'       => __( 'Active Carts', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				"logs" => array(
					'title'       => __( 'Logs', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				"queue" => array(
					'title'       => __( 'Queue', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
				'unsubscribes' => array(
					'title'       => __( 'Unsubscribes', 'automatewoo' ),
					'description' => '',
					'hide_title'  => true,
					'callback'    => array( $this, 'get_report' )
				),
			)
		);

		return $reports;
	}



	/**
	 * Get a report from our reports subfolder
	 */
	public function get_report( $name )
	{
		$name  = sanitize_title( str_replace( '_', '-', $name ) );
		$class = 'AW_Report_' . str_replace( '-', '_', $name );

		include_once AutomateWoo()->plugin_path . 'admin/report-list-table.php';

		include_once( apply_filters( 'automatewoo_admin_reports_path', 'reports/' . $name . '.php', $name, $class ) );

		if ( ! class_exists( $class ) )
			return;

		$report = new $class();
		$report->output_report();
	}


	/**
	 * @param $columns
	 *
	 * @return array
	 */
	function add_columns( $columns )
	{
		if ( aw_request('post_type') != AutomateWoo()->post_type )
			return $columns;

		$date_col = $columns['date'];
		unset( $columns['date'] );

		$columns['times_run'] = __( 'Times Run', 'automatewoo' );
		$columns['queued'] = __( 'Current Queue', 'automatewoo' );

		$columns['date'] = $date_col;

		return $columns;
	}


	/**
	 * @param $column
	 * @param $post_id
	 *
	 * @return array
	 */
	public function add_column_data( $column, $post_id )
	{
		$workflow = new AW_Model_Workflow($post_id);

		if ( ! $workflow->exists )
			return;

		switch ( $column ) {
			case 'times_run':
				echo '<a href="' . admin_url( 'admin.php?page=wc-reports&tab=automatewoo&report=logs&_workflow=' . $workflow->id ) . '">' . $workflow->get_times_run() . '</a>';
				break;
			case 'queued':
				echo $workflow->get_current_queue_count();
				break;

		}
	}



	/**
	 * Adds settings page
	 *
	 * @param $settings
	 * @return mixed|void
	 */
	public function add_settings_page( $settings )
	{
		$settings[] = include( AutomateWoo()->plugin_path . 'admin/settings-page.php' );
	}


	/**
	 * @param $query WP_Query
	 */
	public function set_default_workflow_order($query )
	{
		if ( ! $query->is_main_query() ) return;

		if ( $query->get('post_type') != AutomateWoo()->post_type ) return;

		if ( get_query_var('orderby') ) return;

		$query->set( 'orderby', 'menu_order' );
		$query->set( 'order', 'ASC' );
	}


	/**
	 *
	 */
	public function licence_notices()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) ) return;

		$licence_type = AutomateWoo()->licence->get_type();

		if ( in_array( $licence_type, array( 'live', 'dev' ) ) ) return;

		$licence_page = admin_url('admin.php?page=wc-settings&tab=automatewoo&section=licence');

		if ( $licence_type == 'expired' )
		{
			if ( get_transient('aw_dismiss_licence_expiry_notice') ) return; // notice has been dismissed

			$strong = __( 'Your AutomateWoo licence has expired.', 'automatewoo' );
			$more = sprintf(
				__( '<a href="%s" target="_blank">Renew your licence</a> to receive updates and support.', 'automatewoo' ),
				AutomateWoo()->licence->get_renewal_url(),
				$licence_page
			);

			AutomateWoo()->admin_helper->notice('warning is-dismissible', $strong, $more, 'aw-notice-licence-renew' );
		}
		else
		{
			$strong = __( 'AutomateWoo is not activated.', 'automatewoo' );

			$more = sprintf(
				__( 'Enter your <a href="%s">licence here</a>.', 'automatewoo' ),
				$licence_page
			);

			AutomateWoo()->admin_helper->notice('warning', $strong, $more, 'aw-license-nag' );
		}


	}


}