/**
 * AutomateWoo Workflows Admin
 */


// Remove sortable so it doesn't break wp-editors
jQuery('.meta-box-sortables').removeClass('meta-box-sortables');


jQuery(function($) {

	AutomateWoo.Workflows = {

		$triggers_box: $('#aw_trigger_box'),

		$actions_box: $('#aw_actions_box'),

		$actions_container: $('.aw-actions-container'),

		$action_template: $('.aw-action-template'),

		$trigger_select: $('.js-trigger-select').first(),


		init: function() {
			AutomateWoo.Workflows.init_triggers_box();
			AutomateWoo.Workflows.init_actions_box();
			AutomateWoo.Workflows.init_options_box();
			AutomateWoo.Workflows.init_tooltips();
			AutomateWoo.Workflows.init_text_vars();
			AutomateWoo.Workflows.init_enhanced_selects();

			$( document.body ).on( 'automatewoo-enhanced-select-init', function() {
				AutomateWoo.Workflows.init_enhanced_selects();
			})
		},



		/**
		 *
		 */
		init_triggers_box: function()
		{
			var prev_trigger = AutomateWoo.Workflows.$trigger_select.val();

			AutomateWoo.Workflows.$trigger_select.change(function(){

				var trigger = $(this).val();

				AutomateWoo.Workflows.fill_trigger_fields( trigger, function(){

					if ( AutomateWoo.Workflows.check_actions_are_compatible_with_trigger() ){

						AutomateWoo.Workflows.refine_text_vars();
						AutomateWoo.Workflows.refine_action_selects();
						AutomateWoo.Workflows.maybe_disable_queueing();

						prev_trigger = trigger;

					}
					else {
						//switch back
						AutomateWoo.Workflows.$trigger_select.val( prev_trigger ).trigger('change');
					}

				});
			});
		},


		/**
		 *
		 */
		init_actions_box: function() {


			$('.aw-action.js-open').each(function(){
				AutomateWoo.Workflows.action_edit_open( $(this) );
			});


			$('.js-aw-add-action').click(function (e) {
				e.preventDefault();
				AutomateWoo.Workflows.add_new_action();
			});

			$(document).on('click', '.js-edit-action', function (e) {
				e.preventDefault();

				var $action = $(this).parents('.aw-action').first();

				if ($action.is('.js-open')) {
					AutomateWoo.Workflows.action_edit_close($action);
				}
				else {
					AutomateWoo.Workflows.action_edit_open($action);
				}
			});

			$(document).on('click', '.js-delete-action', function (e) {
				e.preventDefault();
				var $action = $(this).parents('.aw-action').first();
				AutomateWoo.Workflows.action_delete($action);
			});

			// Action select change
			$(document).on('change', '.js-action-select', function () {
				var $action = $(this).parents('.aw-action').first();
				AutomateWoo.Workflows.fill_action_fields( $action, $(this).val() );
			});

			// preview links
			$(document).on('click', '.js-preview-action', function(e){
				e.preventDefault();
				var $action = $(this).parents('.aw-action').first();
				AutomateWoo.Workflows.preview_action($action);
			});


			if ( ! aw_params.is_new_workflow ) {
				AutomateWoo.Workflows.refine_action_selects();
				AutomateWoo.Workflows.refine_text_vars();
				AutomateWoo.Workflows.maybe_disable_queueing();
			}
		},



		init_options_box: function() {

			var $select_when_to_run = $('.js-when-to-run-select'),
				$checkbox_click_tracking = $('.aw-checkbox-enable-click-tracking');

			AutomateWoo.Workflows.maybe_hide_tracking_options();

			$select_when_to_run.change(function(){
				if ( $(this).val() == 'delayed' ) {
					$('.js-when-to-run-delayed').slideDown(150);
				}
				else {
					$('.js-when-to-run-delayed').slideUp(150);
				}

				if ( $(this).val() == 'datetime' ) {
					$('.js-when-to-run-datetime').slideDown(150);
				}
				else {
					$('.js-when-to-run-datetime').slideUp(150);
				}
			});

			$select_when_to_run.trigger('change');

			$checkbox_click_tracking.click(function(){
				AutomateWoo.Workflows.maybe_hide_tracking_options();
			});

		},



		maybe_hide_tracking_options: function() {

			var $checkbox_click_tracking = $('.aw-checkbox-enable-click-tracking');
			var checked = $checkbox_click_tracking.is(':checked');

			if ( ! checked ) {
				$('.js-require-email-tracking').hide();
			}
			else {
				$('.js-require-email-tracking').show();
			}
		},



		init_text_vars: function() {

			$('#aw_text_vars_box code.aw-text-var-outer span.aw-text-var').click(function(){

				AutomateWoo.Workflows.select_text(this);

			});

		},



		/**
		 *
		 * @param trigger_name
		 * @param callback
		 */
		fill_trigger_fields: function( trigger_name, callback )
		{
			AutomateWoo.Workflows.$triggers_box.addClass('loading');
	
			// Remove existing fields
			AutomateWoo.Workflows.$triggers_box.find('tr.aw-trigger-option').remove();
	
			$.ajax({
				method: 'GET',
				url: ajaxurl,
				data: {
					action: 'aw_fill_trigger_fields',
					trigger_name: trigger_name,
					workflow_id: aw_params.workflow_id,
					is_new_workflow: aw_params.is_new_workflow
				}
			})
				.done(function(response){

					if ( ! response.success ) {
						return;
					}

					AutomateWoo.Workflows.$triggers_box.find('tbody').append( response.data.fields );
					AutomateWoo.Workflows.$triggers_box.removeClass('loading');

					AutomateWoo.Workflows.$triggers_box.find('.js-trigger-description').html( response.data.description );

					AutomateWoo.Workflows.init_tooltips();

					if ( callback ) {
						callback();
					}
				})
			;
		},



		add_new_action: function() {

			var $new_action,
				action_number = AutomateWoo.Workflows.get_number_of_actions() + 1;

			$('.js-aw-no-actions-message').hide();

			$new_action = AutomateWoo.Workflows.$action_template.clone();
			$new_action.removeClass('aw-action-template');
			$new_action.addClass('aw-action');

			AutomateWoo.Workflows.$actions_container.append($new_action);

			$new_action.find('._tips').addClass('tips');
			$new_action.attr( 'data-action-number', action_number );

			AutomateWoo.Workflows.action_edit_open($new_action);
		},



		action_edit_open: function( $action ) {

			var action_number = $action.data('action-number');

			$action.addClass('js-open');
			$action.find('.aw-action-fields-container').slideDown(150);
			AutomateWoo.Workflows.init_tooltips();

			// save open state
			$.cookie( 'aw_editing_action_' + aw_params.workflow_id + '_' + action_number , 1);
		},


		action_edit_close: function( $action ) {

			var action_number = $action.data('action-number');

			$action.removeClass('js-open');
			$action.find('.aw-action-fields-container').slideUp(150);

			$.removeCookie( 'aw_editing_action_' + aw_params.workflow_id + '_' + action_number );
		},


		/**
		 * @param $action
		 */
		action_delete: function( $action ) {
			$action.remove();
		},


		/**
		 *
		 */
		fill_action_fields: function( $action, selected_action ) {

			var action_number = $action.data('action-number');
			var $select = $action.find('.js-action-select');

			AutomateWoo.Workflows.$actions_box.addClass('loading');

			// Remove existing fields
			$action.find('tr.aw-field:not([data-name="action_name"])').remove();


			$.ajax({
				method: 'GET',
				url: ajaxurl,
				data: {
					action: 'aw_fill_action_fields',
					action_name: selected_action,
					action_number: action_number,
					workflow_id: aw_params.workflow_id
				}
			})
				.done(function(response){

					$action.find('.aw-table tbody').append( response.data.fields );
					AutomateWoo.Workflows.$actions_box.removeClass('loading');

					// Fill select box name
					$select.attr('name', 'aw_workflow_data[actions]['+action_number+'][action_name]' );

					// Pre fill title
					$action.find('.action-title').text( response.data.title );

					$action.find('.js-action-description').html( response.data.description );

					// show / hide preview link
					if ( AutomateWoo.Workflows.action_can_be_previewed( selected_action ) ) {
						$action.find('.js-preview-action').removeClass('hidden');
					}
					else {
						$action.find('.js-preview-action').addClass('hidden');
					}

					AutomateWoo.Workflows.init_tooltips();

				})
			;

		},


		init_tooltips: function () {
			$('.tips').tipTip({
				'attribute': 'data-tip',
				'fadeIn': 50,
				'fadeOut': 50,
				'delay': 200
			});
		},



		get_number_of_actions: function () {
			return $('.aw-action').length;
		},



		/**
		 * Show or hide text var groups based on the selected trigger
		 */
		refine_text_vars: function()
		{
			$('.aw-text-variables-container > div').each(function(){
				if ( $.inArray( $(this).attr('data-type'), aw_params.supplied_data_items ) == -1 ) {
					$(this).hide();
				}
				else {
					$(this).show();
				}
			});
		},


		/**
		 * Show or hide select options based on the selected trigger
		 * Also what if a trigger is changed after an action is already added
		 */
		refine_action_selects: function()
		{
			$('.js-action-select').each(function(){
				$(this).find('option').each(function(){

					if ( AutomateWoo.Workflows.is_action_compatible_with_current_trigger( $(this).val() ) ) {
						$(this).removeAttr('disabled');
					}
					else {
						$(this).attr('disabled', true);
					}

				});

			});
		},


		/**
		 * Hide queue if disbled for the selected trigger
		 */
		maybe_disable_queueing: function()
		{
			if ( aw_params.current_trigger_allows_queueing ) {
				$('.js-trigger-options-queueing').show();
			}
			else {
				$('.js-trigger-options-queueing').hide();
			}
		},


		/**
		 * Be sure to run this before refine_action_selects
		 *
		 * Returns false if switching back to stop the trigger change
		 *
		 * @return bool
		 */
		check_actions_are_compatible_with_trigger: function(){

			var switch_back = false;

			$('.js-action-select').each(function(){

				if ( ! switch_back && ! AutomateWoo.Workflows.is_action_compatible_with_current_trigger( $(this).val() ) ) {

					var confirm_text,
						title = $(this).find('option:selected').text();

					confirm_text = 'The action ' + title + ' is not compatible with the trigger you selected. Do you want to delete it?';

					if ( confirm( confirm_text ) ) {

						var $action = $(this).parents('.aw-action').first();
						AutomateWoo.Workflows.action_delete( $action );
					}
					else {
						switch_back = true;
					}
				}
			});

			return !switch_back;
		},



		/**
		 *
		 * @param action_name
		 *
		 * @return bool
		 */
		is_action_compatible_with_current_trigger: function( action_name )
		{
			var compatible = true,
				action = aw_params.actions_data[action_name];

			// Not a valid action
			if ( typeof action != 'object' ) {
				return true;
			}

			// No data items required
			if ( ! action.required_data_items.length ){
				return true;
			}

			$.each( action.required_data_items, function(i, value){

				if ( $.inArray( value, aw_params.supplied_data_items ) == -1 ) {
					compatible = false
				}
			});

			return compatible;
		},



		select_text: function( element ) {

			var range;

			if (document.selection) {
				range = document.body.createTextRange();
				range.moveToElementText(element);
				range.select();
			} else if (window.getSelection) {
				range = document.createRange();
				range.selectNode(element);
				window.getSelection().addRange(range);
			}
		},



		preview_action: function( $action ) {

			var request,
				action_number = $action.data('action-number'),
				name_selector,
				preview_data = {};

			AutomateWoo.Workflows.$actions_box.addClass('loading');

			tinyMCE.triggerSave();

			name_selector = 'aw_workflow_data[actions]['+action_number+']';

			// get fields to preview
			$action.find('[name*="' + name_selector + '"]').each(function(){

				// get the name
				var name = $(this).attr('name').replace(name_selector, '').replace('[', '').replace(']', '');

				preview_data[name] = $(this).val();
			});

			request = {
				action: 'aw_email_preview_ui',
				workflow_id: aw_params.workflow_id,
				action_number: action_number
			};

			$.ajax({
				method: 'POST',
				url: ajaxurl,
				data: {
					action: 'aw_save_preview_data',
					workflow_id: aw_params.workflow_id,
					preview_data: preview_data
				},
				success: function(response) {

					var joiner = ajaxurl.indexOf('?') == -1 ? '?' : '&';

					if ( response.success ) {
						window.open( ajaxurl + joiner + $.param(request), 'automatewoo_preview', 'titlebar=no,toolbar=no,height=768,width=800,resizable=yes,status=no');
					}
				},
				complete: function() {
					AutomateWoo.Workflows.$actions_box.removeClass('loading');
				}
			});



		},


		/**
		 * @param action_name
		 * @returns {boolean}
		 */
		action_can_be_previewed: function selected_action( action_name ) {

			var action = aw_params.actions_data[action_name];

			// Not a valid action
			if ( typeof action != 'object' ) {
				return true;
			}

			return action.can_be_previewed;
		},



		init_enhanced_selects: function() {

			// Ajax product search box
			$( ':input.automatewoo-json-search' ).filter( ':not(.enhanced)' ).each( function() {
				var select2_args = {
					allowClear:  $( this ).data( 'allow_clear' ) ? true : false,
					placeholder: $( this ).data( 'placeholder' ),
					minimumInputLength: 1,
					escapeMarkup: function( m ) {
						return m;
					},
					ajax: {
						url:         ajaxurl,
						dataType:    'json',
						quietMillis: 250,
						data: function( term ) {

							var data = {
								term:     term,
								action:   $( this ).data( 'action' ),
								exclude:  $( this ).data( 'exclude' )
							};

							// pass in sibling field data
							var sibling = $(this).data('pass-sibling');

							if ( sibling ) {

								var $sibling = $('[name="'+ sibling+ '"]');

								if ( $sibling.length ) {
									data['sibling'] = $sibling.val()
								}
							}

							return data;
						},
						results: function( data ) {
							var terms = [];
							if ( data ) {
								$.each( data, function( id, text ) {
									terms.push( { id: id, text: text } );
								});
							}
							return {
								results: terms
							};
						},
						cache: true
					}
				};

				if ( $( this ).data( 'multiple' ) === true ) {
					select2_args.multiple = true;
					select2_args.initSelection = function( element, callback ) {
						var data     = $.parseJSON( element.attr( 'data-selected' ) );
						var selected = [];

						$( element.val().split( ',' ) ).each( function( i, val ) {
							selected.push({
								id: val,
								text: data[ val ]
							});
						});
						return callback( selected );
					};
					select2_args.formatSelection = function( data ) {
						return '<div class="selected-option" data-id="' + data.id + '">' + data.text + '</div>';
					};
				} else {
					select2_args.multiple = false;
					select2_args.initSelection = function( element, callback ) {
						var data = {
							id: element.val(),
							text: element.attr( 'data-selected' )
						};
						return callback( data );
					};
				}

				$( this ).select2( select2_args ).addClass( 'enhanced' );
			});

		}




	};
	

	AutomateWoo.Workflows.init();

});