<?php

if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
}

/**
 * AW_Report_List_Table
 *
 * @since 2.0
 */
class AW_Report_List_Table extends WP_List_Table
{
	public $show_tablenav_at_top = false;


	/**
	 * @param array|string $args
	 */
	public function __construct( $args )
	{
		wp_enqueue_script('automatewoo-modal');

		parent::__construct( $args );
	}


	/**
	 * Output the report
	 */
	public function output_report()
	{
		$this->prepare_items();
		echo '<div id="poststuff" class="woocommerce-reports-wide">';
		$this->display();
		echo '</div>';
	}

	/**
	 *
	 */
	public function display_tablenav( $position )
	{
		if ( $position != 'top' || $this->show_tablenav_at_top )
		{
			parent::display_tablenav( $position );
		}
	}


	public function header_note( $text )
	{
		echo '<div class="aw-report-header-note">' . $text . '</div>';
	}



	/**
	 * @param $user
	 */
	public function format_user( $user )
	{
		if ( $user )
			echo "$user->first_name $user->last_name <a href='mailto:$user->user_email'>$user->user_email</a> ";
		else
			$this->format_blank();

	}


	/**
	 * @param $email
	 */
	public function format_guest( $email )
	{
		echo "[Guest] <a href='mailto:$email'>$email</a>";
	}


	/**
	 * @return array
	 */
	protected function get_table_classes()
	{
		return array( 'automatewoo-report-table', 'widefat', 'fixed', 'striped', $this->_args['plural'] );
	}


	/**
	 * @param $date_string
	 * @param bool $is_gmt
	 */
	public function format_date( $date_string, $is_gmt = true )
	{
		if ( $date_string )
			echo aw_display_time( strtotime( $date_string ), false, $is_gmt );
		else
			$this->format_blank();
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 */
	public function format_workflow_title( $workflow )
	{
		if ( ! $workflow->exists )
		{
			$this->format_blank();
		}
		else
		{
			echo '<a href="' . get_edit_post_link( $workflow->id ) . '"><strong>' . $workflow->title . '</strong></a>';

			if ( AutomateWoo()->integrations->is_wpml() )
			{
				echo ' [' . $workflow->get_language() . ']';
			}
		}

	}


	public function format_blank()
	{
		echo '-';
	}

}
