<?php
/**
 * @class		AW_Report_Conversions_List
 * @package		AutomateWoo/Admin/Reports
 */

class AW_Report_Conversions_List extends AW_Report_List_Table
{
	public $_column_headers;

	public $max_items;


	/**
	 * __construct function.
	 */
	public function __construct()
	{
		parent::__construct( array(
			'singular'  => __( 'Conversion', 'automatewoo' ),
			'plural'    => __( 'Conversions', 'automatewoo' ),
			'ajax'      => false
		) );
	}


	/**
	 * No items found text
	 */
	public function no_items()
	{
		_e( 'No conversions found.', 'automatewoo' );
	}



	/**
	 * @param WC_Order $order
	 * @param mixed $column_name
	 */
	public function column_default( $order, $column_name )
	{
		switch( $column_name )
		{
			case 'order':
				echo '<a href="' . get_edit_post_link( $order->id ) . '"><strong>#' . $order->get_order_number() . '</strong></a>';

				break;

			case 'customer':

				if ( $user = $order->get_user() )
					echo '<a href="' . get_edit_user_link( $user->ID ) . '">' . $user->user_firstname . ' ' . $user->user_lastname . '</a>';
				else
					echo $order->billing_first_name . ' ' . $order->billing_last_name;

				break;


			case 'order_placed':

				$this->format_date( $order->order_date, false );

				break;


			case 'log':

				if ( $log_id = get_post_meta( $order->id, '_aw_conversion_log', true ) )
				{
					$url = add_query_arg(array(
						'action' => 'aw_show_log_details',
						'log_id' => $log_id
					), admin_url('admin-ajax.php') );

					echo '<a class="js-open-automatewoo-modal" data-modal-type="ajax" href="' . $url . '">#'.$log_id.'</a>';
				}
				else
				{
					$this->format_blank();
				}

				break;

			case 'interacted':

				if ( $log_id = get_post_meta( $order->id, '_aw_conversion_log', true ) )
				{
					$log = new AW_Model_Log( $log_id );

					$tracking = $log->get_meta('tracking_data');

					if ( is_array( $tracking ) )
					{
						$this->format_date( $tracking[0]['date'] );
					}
				}
				else
				{
					$this->format_blank();
				}

				break;


			case 'workflow':

				$workflow_id = get_post_meta( $order->id, '_aw_conversion', true );

				$this->format_workflow_title( new AW_Model_Workflow( $workflow_id ) );

				break;


			case 'total':

				echo wc_price( $order->get_total() );

				break;

		}
	}

	/**
	 * get_columns function.
	 */
	public function get_columns()
	{
		$columns = array(
			'order'  => __( 'Order', 'automatewoo' ),
			'customer'  => __( 'Customer', 'automatewoo' ),
			'workflow' => __( 'Workflow', 'automatewoo' ),
			'log' => __( 'Log', 'automatewoo' ),
			'interacted' => __( 'First Interacted', 'automatewoo' ),
			'order_placed' => __( 'Order Placed', 'automatewoo' ),
			'total' => __( 'Order Total', 'automatewoo' ),
		);

		return $columns;
	}

	/**
	 * prepare_items function.
	 */
	public function prepare_items()
	{
		$this->_column_headers = array( $this->get_columns(), array(), $this->get_sortable_columns() );
		$current_page = absint( $this->get_pagenum() );
		$per_page = apply_filters( 'automatewoo_report_items_per_page', 20 );

		$this->get_items( $current_page, $per_page );

		/**
		 * Pagination
		 */
		$this->set_pagination_args( array(
			'total_items' => $this->max_items,
			'per_page'    => $per_page,
			'total_pages' => ceil( $this->max_items / $per_page )
		) );
	}



	/**
	 * Get Products matching stock criteria
	 */
	public function get_items( $current_page, $per_page )
	{
		$query = new WP_Query(array(
			'post_type' => 'shop_order',
			'post_status' => array( 'wc-processing', 'wc-completed' ),
			'posts_per_page' => $per_page,
			'offset' => ( $current_page - 1 ) * $per_page,
			'meta_query' => array(
				array(
					'key' => '_aw_conversion',
					'compare' => 'EXISTS',
				)
			)
		));

		foreach ( $query->posts as $order )
		{
			$this->items[] = wc_get_order( $order );
		}

		$this->max_items = $query->found_posts;
	}

}
