<?php
/**
 * @class		AW_Report_Conversions
 * @package		AutomateWoo/Admin/Reports
 */

class AW_Report_Conversions extends WC_Admin_Report
{

	public $chart_colours = array();
	public $workflow_ids = array();
	public $workflow_ids_titles = array();

	public $conversion_orders = array();

	public $conversion_total_value = 0;
	public $conversion_total_orders = 0;


	/**
	 * Constructor
	 */
	public function __construct()
	{
		if ( is_array( aw_request('workflow_ids') ) )
		{
			$this->workflow_ids = array_filter( array_map( 'absint', aw_request('workflow_ids') ) );
		}
		elseif ( aw_request('workflow_ids') )
		{
			$this->workflow_ids = array_filter( array( absint( aw_request('workflow_ids') ) ) );
		}
	}


	/**
	 *
	 */
	public function load_chart_data()
	{
		$start_date = new DateTime();
		$start_date->setTimestamp( $this->start_date );

		$end_date = new DateTime();
		$end_date->setTimestamp( $this->end_date );
		$end_date->modify('+1 days');

		$meta_query = array();

		if ( $this->workflow_ids )
		{
			$meta_query[] = array(
				'key' => '_aw_conversion',
				'value' => $this->workflow_ids,
			);
		}
		else
		{
			$meta_query[] = array(
				'key' => '_aw_conversion',
				'compare' => 'EXISTS',
			);
		}


		// Get converted order
		$orders = new WP_Query(array(
			'post_type' => 'shop_order',
			'post_status' => array( 'wc-processing', 'wc-completed' ),
			'posts_per_page' => -1,
			'fields' => 'ids',
			'meta_query' => $meta_query,
			'date_query' => array(
				array(
					'column' => 'post_date',
					'after' => $start_date->format('Y-m-d H:i:s')
				),
				array(
					'column' => 'post_date',
					'before' => $end_date->format('Y-m-d H:i:s')
				)
			)
		));

		foreach ( $orders->posts as $order_id )
		{
			$order = wc_get_order($order_id);

			$this->conversion_total_value += $order->get_total();

			$order_obj = new stdClass();
			$order_obj->date = $order->post->post_date;
			$order_obj->total = $order->get_total();

			$this->conversion_orders[] = $order_obj;
		}

		$this->conversion_total_orders = $orders->post_count;
	}



	/**
	 * Get the legend for the main chart sidebar
	 * @return array
	 */
	public function get_chart_legend()
	{
		$this->load_chart_data();

		$legend = array();

		$legend[] = array(
			'title' => sprintf( __( '%s converted order value', 'woocommerce' ), '<strong>' . wc_price($this->conversion_total_value) . '</strong>' ),
			'color' => $this->chart_colours['conversion_value'],
			'highlight_series' => 1
		);

		$legend[] = array(
			'title' => sprintf( __( '%s converted orders', 'woocommerce' ), '<strong>' . $this->conversion_total_orders . '</strong>' ),
			'color' => $this->chart_colours['conversion_number'],
			'highlight_series' => 0
		);


		return $legend;
	}


	/**
	 * Output the report
	 */
	public function output_report() {

		$ranges = array(
			'year'         => __( 'Year', 'woocommerce' ),
			'last_month'   => __( 'Last Month', 'woocommerce' ),
			'month'        => __( 'This Month', 'woocommerce' ),
			'7day'         => __( 'Last 7 Days', 'woocommerce' )
		);

		$this->chart_colours = array(
			'conversion_value' => '#3498db',
			'conversion_number' => '#DBE1E3',
		);

		$current_range = ! empty( $_GET['range'] ) ? sanitize_text_field( $_GET['range'] ) : '7day';

		if ( ! in_array( $current_range, array( 'custom', 'year', 'last_month', 'month', '7day' ) ) )
			$current_range = '7day';

		$this->calculate_current_range( $current_range );

		include( WC()->plugin_path() . '/includes/admin/views/html-report-by-date.php');

	}

	/**
	 * [get_chart_widgets description]
	 *
	 * @return array
	 */
	public function get_chart_widgets()
	{

		$widgets = array();

		if ( ! empty( $this->workflow_ids ) ) {
			$widgets[] = array(
				'title'    => __( 'Showing reports for:', 'woocommerce' ),
				'callback' => array( $this, 'current_filters' )
			);
		}

		$widgets[] = array(
			'title'    => '',
			'callback' => array( $this, 'workflows_widget' )
		);

		return $widgets;
	}


	/**
	 * Show current filters
	 */
	public function current_filters()
	{
		$this->workflow_ids_titles = array();

		foreach ( $this->workflow_ids as $workflow_id )
		{
			$workflow = new AW_Model_Workflow( $workflow_id );

			if ( $workflow )
			{
				$this->workflow_ids_titles[] = $workflow->title;
			}
			else
			{
				$this->workflow_ids_titles[] = '#' . $workflow_id;
			}
		}

		echo '<p>' . ' <strong>' . implode( ', ', $this->workflow_ids_titles ) . '</strong></p>';
		echo '<p><a class="button" href="' . esc_url( remove_query_arg( 'workflow_ids' ) ) . '">' . __( 'Reset', 'woocommerce' ) . '</a></p>';
	}

	/**
	 * Workflows selection
	 */
	public function workflows_widget()
	{
		?>
		<h4 class="section_title"><span><?php _e( 'Workflow Search', 'woocommerce' ); ?></span></h4>
		<div class="section">
			<form method="GET">
				<div>
					<input type="hidden" class="wc-product-search" style="width:203px;" name="workflow_ids[]" data-placeholder="<?php _e( 'Search for a workflow&hellip;', 'automatewoo' ); ?>" data-action="aw_json_search_workflows" />
					<input type="submit" class="submit button" value="<?php _e( 'Show', 'woocommerce' ); ?>" />
					<input type="hidden" name="range" value="<?php if ( ! empty( $_GET['range'] ) ) echo esc_attr( $_GET['range'] ) ?>" />
					<input type="hidden" name="start_date" value="<?php if ( ! empty( $_GET['start_date'] ) ) echo esc_attr( $_GET['start_date'] ) ?>" />
					<input type="hidden" name="end_date" value="<?php if ( ! empty( $_GET['end_date'] ) ) echo esc_attr( $_GET['end_date'] ) ?>" />
					<input type="hidden" name="page" value="<?php if ( ! empty( $_GET['page'] ) ) echo esc_attr( $_GET['page'] ) ?>" />
					<input type="hidden" name="tab" value="<?php if ( ! empty( $_GET['tab'] ) ) echo esc_attr( $_GET['tab'] ) ?>" />
					<input type="hidden" name="report" value="<?php if ( ! empty( $_GET['report'] ) ) echo esc_attr( $_GET['report'] ) ?>" />
				</div>
			</form>
		</div>


		<script type="text/javascript">
			jQuery('.section_title').click(function(){
				var next_section = jQuery(this).next('.section');

				if ( jQuery(next_section).is(':visible') )
					return false;

				jQuery('.section:visible').slideUp();
				jQuery('.section_title').removeClass('open');
				jQuery(this).addClass('open').next('.section').slideDown();

				return false;
			});
			jQuery('.section').slideUp( 100, function() {
				jQuery('.section_title:first').click();
			});
		</script>
	<?php
	}

	/**
	 * Output an export link
	 *
	 */
	public function get_export_button()
	{
		$current_range = ! empty( $_GET['range'] ) ? sanitize_text_field( $_GET['range'] ) : '7day';
		?>
		<a
			href="#"
			download="automatewoo-report-email-tracking-<?php echo esc_attr( $current_range ); ?>-<?php echo date_i18n( 'Y-m-d', current_time('timestamp') ); ?>.csv"
			class="export_csv"
			data-export="chart"
			data-xaxes="<?php _e( 'Date', 'woocommerce' ); ?>"
			data-groupby="<?php echo $this->chart_groupby; ?>"
			>
			<?php _e( 'Export CSV', 'woocommerce' ); ?>
		</a>
	<?php
	}

	/**
	 * Get the main chart
	 *
	 * @return string
	 */
	public function get_main_chart()
	{
		global $wp_locale;

		// Prepare data for report
		$conversion_value = $this->prepare_chart_data( $this->conversion_orders, 'date', 'total', $this->chart_interval, $this->start_date, $this->chart_groupby );
		$conversion_number = $this->prepare_chart_data( $this->conversion_orders, 'date', false, $this->chart_interval, $this->start_date, $this->chart_groupby );

		// Encode in json format
		$chart_data = json_encode(array(
			'conversion_value' => array_values( $conversion_value ),
			'conversion_number' => array_values( $conversion_number ),
		));

		?>
		<div class="chart-container">
			<div class="chart-placeholder main"></div>
		</div>
		<script type="text/javascript">
			var main_chart;

			jQuery(function(){

				var order_data = jQuery.parseJSON( '<?php echo $chart_data; ?>' );

				var drawGraph = function( highlight ) {

					var series = [
						{
							label: "<?php echo esc_js( __( 'Conversion Number', 'automatewoo' ) ) ?>",
							data: order_data.conversion_number,
							yaxis: 1,
							color: '<?php echo $this->chart_colours['conversion_number']; ?>',
							bars: { fillColor: '<?php echo $this->chart_colours['conversion_number']; ?>', fill: true, show: true, lineWidth: 0, barWidth: 60 * 60 * 24 * 1000, align: 'center' },
							shadowSize: 0,
							hoverable: false
						},
						{
							label: "<?php echo esc_js( __( 'Conversion Value', 'automatewoo' ) ) ?>",
							data: order_data.conversion_value,
							yaxis: 2,
							color: '<?php echo $this->chart_colours['conversion_value']; ?>',
							points: { show: true, radius: 5, lineWidth: 3, fillColor: '#fff', fill: true },
							lines: { show: true, lineWidth: 4, fill: false },
							shadowSize: 0
						},
					];

				if ( highlight !== 'undefined' && series[ highlight ] ) {
					highlight_series = series[ highlight ];

					highlight_series.color = '#9c5d90';

					if ( highlight_series.bars )
						highlight_series.bars.fillColor = '#9c5d90';

					if ( highlight_series.lines ) {
						highlight_series.lines.lineWidth = 5;
					}
				}

			main_chart = jQuery.plot(
				jQuery('.chart-placeholder.main'),
				series,
				{
					legend: {
						show: false
					},
					grid: {
						color: '#aaa',
						borderColor: 'transparent',
						borderWidth: 0,
						hoverable: true
					},
					xaxes: [ {
						color: '#aaa',
						position: "bottom",
						tickColor: 'transparent',
						mode: "time",
						timeformat: "<?php if ( $this->chart_groupby == 'day' ) echo '%d %b'; else echo '%b'; ?>",
						monthNames: <?php echo json_encode( array_values( $wp_locale->month_abbrev ) ) ?>,
						tickLength: 1,
						minTickSize: [1, "<?php echo $this->chart_groupby; ?>"],
						font: {
							color: "#aaa"
						}
					} ],
					yaxes: [
						{
							min: 0,
							minTickSize: 1,
							tickDecimals: 0,
							color: '#d4d9dc',
							font: { color: "#aaa" }
						},
						{
							position: "right",
							min: 0,
							tickDecimals: 0,
							alignTicksWithAxis: 0,
							color: '#eee',
							font: { color: "#aaa" }
						}
					]
				}
			);

			jQuery('.chart-placeholder').resize();
			}

			drawGraph();

			jQuery('.highlight_series').hover(
				function() {
					drawGraph( jQuery(this).data('series') );
				},
				function() {
					drawGraph();
				}
			);
			});
		</script>
	<?php

	}

}
