<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * AW_Report_Logs
 */
class AW_Report_Logs extends AW_Report_List_Table
{
	public $_column_headers;
	public $max_items;
	public $show_tablenav_at_top = true;


	/**
	 * __construct function.
	 */
	public function __construct()
	{
		parent::__construct( array(
			'singular'  => __( 'Log', 'automatewoo' ),
			'plural'    => __( 'Logs', 'automatewoo' ),
			'ajax'      => false
		) );
	}



	/**
	 * Display the bulk actions dropdown.
	 *
	 * @since 3.1.0
	 * @access protected
	 *
	 * @param string $which The location of the bulk actions: 'top' or 'bottom'.
	 *                      This is designated as optional for backwards-compatibility.
	 */
	protected function bulk_actions( $which = '' )
	{
		if ( $which == 'top' )
		{
			$user_string = '';
			$user_id = '';
			$workflow_id = '';
			$workflow_name = '';

			if ( ! empty( $_GET['_workflow'] ) )
			{
				$workflow_id = absint( $_GET['_workflow'] );
				$workflow_name = get_the_title( $_GET['_workflow'] );
			}

			if ( ! empty( $_GET['_customer_user'] ) )
			{
				$user_id     = absint( $_GET['_customer_user'] );
				$user        = get_user_by( 'id', $user_id );
				$user_string = esc_html( $user->display_name ) . ' (#' . absint( $user->ID ) . ' &ndash; ' . esc_html( $user->user_email );
			}
			?>
			<form method="GET">

				<input type="hidden" name="page" value="<?php if ( ! empty( $_GET['page'] ) ) echo esc_attr( $_GET['page'] ) ?>" />
				<input type="hidden" name="tab" value="<?php if ( ! empty( $_GET['tab'] ) ) echo esc_attr( $_GET['tab'] ) ?>" />
				<input type="hidden" name="report" value="<?php if ( ! empty( $_GET['report'] ) ) echo esc_attr( $_GET['report'] ) ?>" />

				<input type="hidden" class="wc-product-search" style="width:203px;" name="_workflow"
				       data-placeholder="<?php _e( 'Search for a workflow&hellip;', 'automatewoo' ); ?>"
				       data-selected="<?php echo htmlspecialchars( $workflow_name ); ?>" value="<?php echo $workflow_id; ?>"
				       data-action="aw_json_search_workflows" data-allow_clear="true">

				<input type="hidden" class="wc-customer-search" name="_customer_user"
				       data-placeholder="<?php esc_attr_e( 'Search for a customer&hellip;', 'automatewoo' ); ?>"
				       data-selected="<?php echo htmlspecialchars( $user_string ); ?>" value="<?php echo $user_id; ?>"
				       data-allow_clear="true">

				<?php submit_button( __( 'Filter' ), 'button', 'submit', false ); ?>
			</form>
		<?php
		}
	}




	/**
	 * No items found text
	 */
	public function no_items()
	{
		_e( 'No logs found.', 'automatewoo' );
	}



	/**
	 * column_default function.
	 *
	 * @param AW_Model_Log $log
	 * @param mixed $column_name
	 */
	public function column_default( $log, $column_name )
	{
		switch( $column_name )
		{
			case 'id':
				echo '#' . $log->id;
				break;

			case 'workflow':

				$this->format_workflow_title( $log->get_workflow() );

				break;

			case 'user':

				if ( $email = $log->get_meta('guest_email') )
				{
					$this->format_guest($email);
				}
				elseif ( $user_id = $log->get_meta('user_id') )
				{
					$user = get_user_by( 'id', $user_id );
					$this->format_user($user);
				}
				else
				{
					$this->format_blank();
				}
				break;

			case 'time':
				$this->format_date( $log->date );
				break;

			case 'actions':

				$url = add_query_arg(array(
					'action' => 'aw_show_log_details',
					'log_id' => $log->id
					), admin_url('admin-ajax.php') );
				echo '<a class="button view aw-button-icon js-open-automatewoo-modal" data-modal-type="ajax" href="' . $url . '">View</a>';

				break;
		}
	}

	/**
	 * get_columns function.
	 */
	public function get_columns()
	{
		$columns = array(
			'id'  => __( 'Log', 'automatewoo' ),
			'workflow'  => __( 'Workflow', 'automatewoo' ),
			'user' => __( 'User', 'automatewoo' ),
			'time' => __( 'Time', 'automatewoo' ),
			'actions' => '',
		);

		return $columns;
	}

	/**
	 * prepare_items function.
	 */
	public function prepare_items()
	{
		$this->_column_headers = array( $this->get_columns(), array(), $this->get_sortable_columns() );
		$current_page = absint( $this->get_pagenum() );
		$per_page = apply_filters( 'automatewoo_report_items_per_page', 20 );

		$this->get_items( $current_page, $per_page );

		/**
		 * Pagination
		 */
		$this->set_pagination_args( array(
			'total_items' => $this->max_items,
			'per_page'    => $per_page,
			'total_pages' => ceil( $this->max_items / $per_page )
		) );
	}



	/**
	 * Get Products matching stock criteria
	 */
	public function get_items( $current_page, $per_page )
	{
		$query = new AW_Query_Logs();
		$query->set_limit( $per_page );
		$query->set_offset( ($current_page - 1 ) * $per_page );
		$query->set_ordering('date', 'DESC');

		if ( ! empty($_GET['_workflow']) )
		{
			$query->where('workflow_id', absint( $_GET['_workflow'] ) );
		}

		if ( ! empty($_GET['_customer_user']) )
		{
			$query->where('user_id', absint($_GET['_customer_user']) );
		}

		$res = $query->get_results();

		$this->items = $res;

		$this->max_items = $query->found_rows;

	}

}
