<?php
/**
 * @class		AW_Report_Queue
 * @package		AutomateWoo/Admin/Reports
 */

class AW_Report_Queue extends AW_Report_List_Table
{
	public $_column_headers;
	public $max_items;

	/**
	 * __construct function.
	 */
	public function __construct()
	{
		parent::__construct( array(
			'singular'  => __( 'Event', 'automatewoo' ),
			'plural'    => __( 'Events', 'automatewoo' ),
			'ajax'      => false
		) );
	}

	/**
	 * No items found text
	 */
	public function no_items()
	{
		_e( 'Nothing is queued right now.', 'automatewoo' );
	}


	/**
	 * Output the report
	 */
	public function output_report()
	{
		// Manual delete a queued event
		if ( ! empty( $_GET['delete'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'delete' ) )
		{
			$queued_run = new AW_Model_Queued_Event( absint( $_GET['delete'] ) );
			$queued_run->delete();

			echo '<div class="notice notice-success"><p>' . __( 'Queued Event Deleted', 'automatewoo' ) . '</p></div>';
		}


		// Manual run a queued event
		if ( ! empty( $_GET['run-now'] ) && wp_verify_nonce( $_REQUEST['_wpnonce'], 'run-now' ) )
		{
			$event = new AW_Model_Queued_Event( absint( $_GET['run-now'] ) );

			if ( $event->run() )
			{
				echo '<div class="notice notice-success"><p>' . __( 'Queued Event Run', 'automatewoo' ) . '</p></div>';
			}
			else
			{
				echo '<div class="notice notice-error"><p>' . __( 'Queued event could not be run', 'automatewoo' ) . '</p></div>';
			}
		}

		$this->header_note(
			__( 'Queued events are checked for at 15 minute intervals so the actual run time may vary.', 'automatewoo' )
		);

		parent::output_report();

	}


	/**
	 * column_default function.
	 *
	 * @param $item AW_Model_Queued_Event
	 * @param mixed $column_name
	 */
	public function column_default( $item, $column_name )
	{
		$workflow = $item->get_workflow();

		switch( $column_name )
		{
			case 'queued_event_id':
				echo '#' . $item->id . '';
				echo ( $item->failed ? '<span class="error-message"> - Failed</span>' : '' );
				break;

			case 'workflow':
				$this->format_workflow_title( $workflow );
				break;


			case 'user':

				if ( $workflow )
				{
					if ( $user = $workflow->get_data_item('user') )
					{
						$this->format_user( $user );
					}
					elseif( $guest = $workflow->get_data_item('guest') )
					{
						$this->format_guest( $guest->email );
					}
					else
					{
						$this->format_blank();
					}
				}

				break;

			case 'date':

				$this->format_date( $item->date );

				break;

			case 'actions':

				?>
					<a class="button" href="<?php echo wp_nonce_url( add_query_arg( 'run-now', $item->id ), 'run-now' ); ?>">Run now</a>
					<a class="button" href="<?php echo wp_nonce_url( add_query_arg( 'delete', $item->id ), 'delete' ); ?>">Delete</a>
				<?php

				break;

		}
	}

	/**
	 * get_columns function.
	 */
	public function get_columns()
	{
		$columns = array(
			'queued_event_id' => __( 'Queued Event', 'automatewoo' ),
			'workflow' => __( 'Workflow', 'automatewoo' ),
			'user' => __( 'User', 'automatewoo' ),
			'date' => __( 'Run Date', 'automatewoo' ),
			'actions' => __( 'Actions', 'automatewoo' ),
		);

		return $columns;
	}

	/**
	 * prepare_items function.
	 */
	public function prepare_items()
	{
		$this->_column_headers = array( $this->get_columns(), array(), $this->get_sortable_columns() );
		$current_page = absint( $this->get_pagenum() );
		$per_page = apply_filters( 'automatewoo_report_items_per_page', 20 );

		$this->get_items( $current_page, $per_page );

		/**
		 * Pagination
		 */
		$this->set_pagination_args( array(
			'total_items' => $this->max_items,
			'per_page'    => $per_page,
			'total_pages' => ceil( $this->max_items / $per_page )
		) );
	}



	/**
	 * Get Products matching stock criteria
	 */
	public function get_items( $current_page, $per_page )
	{
		$query = new AW_Query_Queue();
		$query->set_limit( $per_page );
		$query->set_offset( ($current_page - 1 ) * $per_page );
		$query->set_ordering('date', 'ASC');
		$res = $query->get_results();

		$this->items = $res;

		$this->max_items = $query->found_rows;

	}

}
