<?php
/**
 * AW_Report_Runs_By_Date
 */
class AW_Report_Runs_By_Date extends WC_Admin_Report
{

	public $chart_colours = array();
	public $workflow_ids = array();
	public $workflow_ids_titles = array();

	public $logs;
	public $logs_count = 0;


	/**
	 * Constructor
	 */
	public function __construct()
	{
		if ( is_array( aw_request('workflow_ids') ) )
		{
			$this->workflow_ids = array_filter( array_map( 'absint', aw_request('workflow_ids') ) );
		}
		elseif ( aw_request('workflow_ids') )
		{
			$this->workflow_ids = array_filter( array( absint( aw_request('workflow_ids') ) ) );
		}

	}


	/**
	 *
	 */
	public function load_chart_data()
	{
		// Get logs
		$logs_query = new AW_Query_Logs();

		if ( ! empty($this->workflow_ids) )
			$logs_query->where( 'workflow_id', $this->workflow_ids );

		$start_date = new DateTime();
		$start_date->setTimestamp( $this->start_date );

		$end_date = new DateTime();
		$end_date->setTimestamp( $this->end_date );
		$end_date->modify('+1 days');

		// convert to UTC
		$gmt_start_date = get_gmt_from_date( $start_date->format('Y-m-d H:i:s') );
		$gmt_end_date = get_gmt_from_date( $end_date->format('Y-m-d H:i:s') );

		$logs_query->where('date', $gmt_start_date, '>');
		$logs_query->where('date', $gmt_end_date, '<');

		$this->logs = $logs_query->get_results();

		if ( $this->logs )
			$this->logs_count = count( $this->logs );

	}


	/**
	 * Get the legend for the main chart sidebar
	 * @return array
	 */
	public function get_chart_legend()
	{
		$this->load_chart_data();

		$legend = array();

		$legend[] = array(
			'title' => sprintf( __( '%s workflows have run for the selected period', 'woocommerce' ), '<strong>' . $this->logs_count . '</strong>' ),
			'color' => $this->chart_colours['runs'],
			'highlight_series' => 0
		);

		return $legend;
	}


	/**
	 * Output the report
	 */
	public function output_report() {

		$ranges = array(
			'year'         => __( 'Year', 'woocommerce' ),
			'last_month'   => __( 'Last Month', 'woocommerce' ),
			'month'        => __( 'This Month', 'woocommerce' ),
			'7day'         => __( 'Last 7 Days', 'woocommerce' )
		);

		$this->chart_colours = array(
			'runs' => '#3498db',
		);

		$current_range = ! empty( $_GET['range'] ) ? sanitize_text_field( $_GET['range'] ) : '7day';

		if ( ! in_array( $current_range, array( 'custom', 'year', 'last_month', 'month', '7day' ) ) )
			$current_range = '7day';

		$this->calculate_current_range( $current_range );

		include( WC()->plugin_path() . '/includes/admin/views/html-report-by-date.php');

	}

	/**
	 * [get_chart_widgets description]
	 *
	 * @return array
	 */
	public function get_chart_widgets() {

		$widgets = array();

		if ( ! empty( $this->workflow_ids ) ) {
			$widgets[] = array(
				'title'    => __( 'Showing reports for:', 'woocommerce' ),
				'callback' => array( $this, 'current_filters' )
			);
		}

		$widgets[] = array(
			'title'    => '',
			'callback' => array( $this, 'workflows_widget' )
		);

		return $widgets;
	}

	/**
	 * Show current filters
	 */
	public function current_filters()
	{
		$this->workflow_ids_titles = array();

		foreach ( $this->workflow_ids as $workflow_id )
		{
			$workflow = new AW_Model_Workflow( $workflow_id );

			if ( $workflow )
			{
				$this->workflow_ids_titles[] = $workflow->title;
			}
			else
			{
				$this->workflow_ids_titles[] = '#' . $workflow_id;
			}
		}

		echo '<p>' . ' <strong>' . implode( ', ', $this->workflow_ids_titles ) . '</strong></p>';
		echo '<p><a class="button" href="' . esc_url( remove_query_arg( 'workflow_ids' ) ) . '">' . __( 'Reset', 'woocommerce' ) . '</a></p>';
	}

	/**
	 * Product selection
	 */
	public function workflows_widget() {
		?>
		<h4 class="section_title"><span><?php _e( 'Workflow Search', 'woocommerce' ); ?></span></h4>
		<div class="section">
			<form method="GET">
				<div>
					<input type="hidden" class="wc-product-search" style="width:203px;" name="workflow_ids[]" data-placeholder="<?php _e( 'Search for a workflow&hellip;', 'automatewoo' ); ?>" data-action="aw_json_search_workflows" />
					<input type="submit" class="submit button" value="<?php _e( 'Show', 'woocommerce' ); ?>" />
					<input type="hidden" name="range" value="<?php if ( ! empty( $_GET['range'] ) ) echo esc_attr( $_GET['range'] ) ?>" />
					<input type="hidden" name="start_date" value="<?php if ( ! empty( $_GET['start_date'] ) ) echo esc_attr( $_GET['start_date'] ) ?>" />
					<input type="hidden" name="end_date" value="<?php if ( ! empty( $_GET['end_date'] ) ) echo esc_attr( $_GET['end_date'] ) ?>" />
					<input type="hidden" name="page" value="<?php if ( ! empty( $_GET['page'] ) ) echo esc_attr( $_GET['page'] ) ?>" />
					<input type="hidden" name="tab" value="<?php if ( ! empty( $_GET['tab'] ) ) echo esc_attr( $_GET['tab'] ) ?>" />
					<input type="hidden" name="report" value="<?php if ( ! empty( $_GET['report'] ) ) echo esc_attr( $_GET['report'] ) ?>" />
				</div>
			</form>
		</div>


		<script type="text/javascript">
			jQuery('.section_title').click(function(){
				var next_section = jQuery(this).next('.section');

				if ( jQuery(next_section).is(':visible') )
					return false;

				jQuery('.section:visible').slideUp();
				jQuery('.section_title').removeClass('open');
				jQuery(this).addClass('open').next('.section').slideDown();

				return false;
			});
			jQuery('.section').slideUp( 100, function() {
				jQuery('.section_title:first').click();
			});
		</script>
	<?php
	}


	/**
	 * Output an export link
	 */
	public function get_export_button()
	{
		$current_range = ! empty( $_GET['range'] ) ? sanitize_text_field( $_GET['range'] ) : '7day';
		?>
		<a
			href="#"
			download="automatewoo-report-<?php echo esc_attr( $current_range ); ?>-<?php echo date_i18n( 'Y-m-d', current_time('timestamp') ); ?>.csv"
			class="export_csv"
			data-export="chart"
			data-xaxes="<?php _e( 'Date', 'woocommerce' ); ?>"
			data-groupby="<?php echo $this->chart_groupby; ?>"
			>
			<?php _e( 'Export CSV', 'woocommerce' ); ?>
		</a>
		<?php
	}


	/**
	 * Get the main chart
	 *
	 * @return string
	 */
	public function get_main_chart()
	{
		global $wp_locale;

		$logs = $this->logs;

		if ( ! is_array( $logs ) )
			$logs = array();

		// convert all dates to site time
		foreach ( $logs as $log )
		{
			$log->_date_site_time = get_date_from_gmt( $log->date );
		}

		// Prepare data for report
		$log  = $this->prepare_chart_data( $logs, '_date_site_time', false, $this->chart_interval, $this->start_date, $this->chart_groupby );

		// Encode in json format
		$chart_data = json_encode( array(
			'logs'  => array_values( $log ),
		));

		?>
		<div class="chart-container">
			<div class="chart-placeholder main"></div>
		</div>
		<script type="text/javascript">
			var main_chart;

			jQuery(function(){
				var order_data = jQuery.parseJSON( '<?php echo $chart_data; ?>' );

				var drawGraph = function( highlight ) {

					var series = [
					{
						label: "<?php echo esc_js( __( 'Runs', 'automatewoo' ) ) ?>",
						data: order_data.logs,
						yaxis: 2,
						color: '<?php echo $this->chart_colours['runs']; ?>',
						points: { show: true, radius: 5, lineWidth: 3, fillColor: '#fff', fill: true },
						lines: { show: true, lineWidth: 4, fill: false },
						shadowSize: 0
					}
				];

				if ( highlight !== 'undefined' && series[ highlight ] ) {
					highlight_series = series[ highlight ];

					highlight_series.color = '#9c5d90';

					if ( highlight_series.bars )
						highlight_series.bars.fillColor = '#9c5d90';

					if ( highlight_series.lines ) {
						highlight_series.lines.lineWidth = 5;
					}
				}

			main_chart = jQuery.plot(
				jQuery('.chart-placeholder.main'),
				series,
				{
					legend: {
						show: false
					},
					grid: {
						color: '#aaa',
						borderColor: 'transparent',
						borderWidth: 0,
						hoverable: true
					},
					xaxes: [ {
						color: '#aaa',
						position: "bottom",
						tickColor: 'transparent',
						mode: "time",
						timeformat: "<?php if ( $this->chart_groupby == 'day' ) echo '%d %b'; else echo '%b'; ?>",
						monthNames: <?php echo json_encode( array_values( $wp_locale->month_abbrev ) ) ?>,
						tickLength: 1,
						minTickSize: [1, "<?php echo $this->chart_groupby; ?>"],
						font: {
							color: "#aaa"
						}
					} ],
					yaxes: [
						{
							min: 0,
							minTickSize: 10,
							tickDecimals: 0,
							color: '#fff',
							font: { color: "#fff" }
						},
						{
							//position: "right",
							min: 0,
							tickDecimals: 0,
							alignTicksWithAxis: 0,
							color: '#eee',
							font: { color: "#aaa" }
						}
					],
				}
			);

			jQuery('.chart-placeholder').resize();
			}

			drawGraph();

			jQuery('.highlight_series').hover(
				function() {
					drawGraph( jQuery(this).data('series') );
				},
				function() {
					drawGraph();
				}
			);
			});
		</script>
	<?php

	}

}
