<?php

if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * @class AW_Settings_Page
 */

class AW_Settings_Page extends WC_Settings_Page
{

	/**
	 * Constructor.
	 */
	public function __construct()
	{
		$this->id    = 'automatewoo';
		$this->label = __( 'AutomateWoo', 'automatewoo' );

		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_page' ), 20 );
		add_action( 'woocommerce_settings_' . $this->id, array( $this, 'output' ) );
		add_action( 'woocommerce_settings_save_' . $this->id, array( $this, 'save' ) );
		add_action( 'woocommerce_sections_' . $this->id, array( $this, 'output_sections' ) );

	}


	/**
	 * Get sections
	 *
	 * @return array
	 */
	public function get_sections()
	{
		$sections = array(
			'' => __( 'General', 'automatewoo' ),
			'mailchimp' => __( 'MailChimp', 'automatewoo' ),
			'twilio' => __( 'Twilio', 'automatewoo' ),
			'active_campaign' => __( 'ActiveCampaign', 'automatewoo' ),
			'licence' => __( 'Licence', 'automatewoo' ),
			'system_check' => __( 'System Check', 'automatewoo' ),
			'tools' => __( 'Tools', 'automatewoo' ),
		);

		return apply_filters( 'woocommerce_get_sections_' . $this->id, $sections );
	}



	/**
	 * Output the settings
	 */
	public function output()
	{
		global $current_section, $hide_save_button;


		switch ( $current_section )
		{
			case 'licence':

				$hide_save_button = true;

				AutomateWoo()->licence->check_for_domain_mismatch();
				AutomateWoo()->licence->check_licence_status();

				AutomateWoo()->admin_helper->get_view('settings-licence-form');

				wp_enqueue_script('automatewoo-activate');

				break;

			case 'twilio':

				$settings = $this->get_settings( $current_section );
				WC_Admin_Settings::output_fields( $settings );

				$hide_save_button = true;
				AutomateWoo()->admin_helper->get_view('settings-submit-btn');
				AutomateWoo()->admin_helper->get_view('sms-test-twilio');

				wp_enqueue_script('automatewoo-sms-test');

				break;

			case 'tools':

				$hide_save_button = true;
				AutomateWoo()->admin_helper->get_view('settings-tools');
				wp_enqueue_script('automatewoo-tools');

				break;

			case 'system_check':

				AutomateWoo()->admin_helper->get_view('system-check');

				$settings = $this->get_settings( $current_section );
				WC_Admin_Settings::output_fields( $settings );

				break;

			default:

				$settings = $this->get_settings( $current_section );
				WC_Admin_Settings::output_fields( $settings );

				break;
		}

	}



	/**
	 * Save settings
	 */
	public function save()
	{
		global $current_section;

		if ( 'licence' != $current_section )
		{
			$settings = $this->get_settings( $current_section );
			WC_Admin_Settings::save_fields( $settings );
		}

	}



	/**
	 * @param string $current_section
	 *
	 * @return mixed|void
	 */
	public function get_settings( $current_section = '' )
	{
		if ( 'twilio' == $current_section )
		{
			$settings = array(

				array(
					'title' => __( 'Twilio Integration', 'automatewoo' ),
					'type' 	=> 'title',
					'id' 	=> 'automatewoo_twilio_integration'
				),

				array(
					'title'         => __( 'Enable', 'woocommerce' ),
					'id'            => 'automatewoo_twilio_integration_enabled',
					'desc' 	=> __( 'Enable Twilio Integration', 'automatewoo' ),
					'default'       => 'no',
					'autoload' => true,
					'type'          => 'checkbox',
				),

				array(
					'title'    => __( 'From', 'automatewoo' ),
					'desc'     =>  __( 'Must be a Twilio phone number (in E.164 format) or alphanumeric sender ID.', 'automatewoo' ),
					'id'       => 'automatewoo_twilio_from',
					'type'     => 'text',
					'autoload' => false,
					'css'      => 'min-width:300px;',
				),

				array(
					'title'    => __( 'Account SID', 'automatewoo' ),
					'id'       => 'automatewoo_twilio_auth_id',
					'type'     => 'text',
					'autoload' => false,
					'css'      => 'min-width:300px;',
				),

				array(
					'title'    => __( 'Auth Token', 'automatewoo' ),
					'id'       => 'automatewoo_twilio_auth_token',
					'type'     => 'text',
					'autoload' => false,
					'css'      => 'min-width:300px;',
				),

				array(
					'type' 	=> 'sectionend',
					'id' 	=> 'automatewoo_twilio_integration'
				),

			);
		}
		elseif ( 'active_campaign' == $current_section )
		{

			$settings = array(

				array(
					'title' => __( 'ActiveCampaign Integration', 'automatewoo' ),
					'type' 	=> 'title',
					'id' 	=> 'automatewoo_active_campaign_integration'
				),

				array(
					'title'         => __( 'Enable', 'woocommerce' ),
					'id'            => 'automatewoo_active_campaign_integration_enabled',
					'desc' 	=> __( 'Enable ActiveCampaign Integration', 'automatewoo' ),
					'default'       => 'no',
					'autoload' => true,
					'type'          => 'checkbox',
				),

				array(
					'title'    => __( 'API URL', 'automatewoo' ),
					'id'       => 'automatewoo_active_campaign_api_url',
					'type'     => 'text',
					'autoload' => false,
					'css'      => 'min-width:300px;',
				),

				array(
					'title'    => __( 'API Key', 'automatewoo' ),
					'id'       => 'automatewoo_active_campaign_api_key',
					'type'     => 'text',
					'autoload' => false,
					'css'      => 'min-width:300px;',
				),

				array(
					'type' 	=> 'sectionend',
					'id' 	=> 'automatewoo_active_campaign_integration'
				),

			);
		}
		elseif ( 'mailchimp' == $current_section )
		{

			$settings = array(

				array(
					'title' => __( 'MailChimp Integration', 'automatewoo' ),
					'type' 	=> 'title',
					'id' 	=> 'automatewoo_mailchimp_integration'
				),

				array(
					'title'         => __( 'Enable', 'woocommerce' ),
					'id'            => 'automatewoo_mailchimp_integration_enabled',
					'desc' 	=> __( 'Enable MailChimp Integration', 'automatewoo' ),
					'default'       => 'no',
					'autoload' => true,
					'type'          => 'checkbox',
				),

				array(
					'title'    => __( 'API Key', 'automatewoo' ),
					'id'       => 'automatewoo_mailchimp_api_key',
					'desc' 	=> __( 'You can get your API key when logged in to MailChimp under Account > Extras > API Keys.', 'automatewoo' ),
					'type'     => 'text',
					'autoload' => false,
					'css'      => 'min-width:300px;',
				),

				array(
					'type' 	=> 'sectionend',
					'id' 	=> 'automatewoo_mailchimp_integration'
				),

			);
		}
//		elseif ( 'url_shortener' == $current_section )
//		{
//			$settings = array(
//
//				array(
//					'title' => __( 'URL Shortener', 'automatewoo' ),
//					'desc' => __( 'The Google URL Shortener (goo.gl) can be used avoid long URLs the body of an SMS.', 'automatewoo' ),
//					'type' 	=> 'title',
//					'id' 	=> 'automatewoo_shortener_integration'
//				),
//
//				array(
//					'title'         => __( 'Enable', 'woocommerce' ),
//					'id'            => 'automatewoo_url_shortener_enabled',
//					'desc' 	=> __( 'Enable Google URL Shortener', 'automatewoo' ),
//					'default'       => 'no',
//					'autoload' => true,
//					'type'          => 'checkbox',
//				),
//
//				array(
//					'title'    => __( 'Google API Key', 'automatewoo' ),
//					'id'       => 'automatewoo_google_api_key',
//					'desc' => sprintf(
//						__( 'Create a <a href="%s" target="_blank">Google API Key</a>.', 'automatewoo' ),
//						'https://code.google.com/apis/console/'
//					),
//					'type'     => 'text',
//					'autoload' => false,
//					'css'      => 'min-width:300px;',
//				),
//
//				array(
//					'type' 	=> 'sectionend',
//					'id' 	=> 'automatewoo_shortener_integration'
//				),
//
//			);
//		}
		elseif( 'system_check' == $current_section )
		{
			$settings = array(

				array(
					'title' => '',
					'type' 	=> 'title',
					'id' 	=> 'automatewoo_system_check_options'
				),

				array(
					'title'         => __( 'Enable Background Checks', 'woocommerce' ),
					'id'            => 'automatewoo_enable_background_system_check',
					'desc' 	=> __( 'Allow occasional background checks for major system issues. If an issue is detected an admin notice will appear.', 'automatewoo' ),
					'default'       => 'yes',
					'autoload' => true,
					'type'          => 'checkbox',
				),

				array(
					'type' 	=> 'sectionend',
					'id' 	=> 'automatewoo_system_check_options'
				),

			);
		}
		else
		{
			$settings = array(

				array(
					'title' => __( 'General Settings', 'automatewoo' ),
					'type' 	=> 'title',
					'id' 	=> 'automatewoo_general'
				),

				array(
					'title'         => __( 'Where should guest email capture be enabled?', 'automatewoo' ),
					'id'            => 'automatewoo_guest_email_capture_scope',
					'desc' 	=>
						__( "Determines which pages have javascript code inserted for email capture. If unsure leave set to Checkout Only.", 'automatewoo' ) . '<br><br>'
						. __("If set to All Pages you can add the CSS class 'automatewoo-capture-guest-email' to enable email capture on custom form fields.", 'automatewoo' ),
					'default'       => 'checkout',
					'autoload' => true,
					'type'          => 'select',
					'options' => array(
						'checkout' => __( 'Checkout Only', 'automatewoo' ),
						'all' => __( 'All Pages', 'automatewoo' ),
						'none' => __( 'No Pages', 'automatewoo' ),
					),
					'desc_tip' => true,
				),

				array(
					'title'    => __( 'Queue Batch Size', 'automatewoo' ),
					'id'       => 'automatewoo_queue_batch_size',
					'desc' 	=> __( 'Sets the maximum number of queued events to process at once. The queue runs every 15 minutes. Default value is 25.', 'automatewoo' ),
					'type'     => 'number',
					'placeholder' => '25',
					'autoload' => false,
				),

				array(
					'title'    => __( 'Conversion Tracking Window', 'automatewoo' ),
					'id'       => 'automatewoo_conversion_window',
					'desc' 	=> __( 'Sets the number of days after a workflow runs that a new order can be considered a conversion. Default value is 14.', 'automatewoo' ),
					'type'     => 'number',
					'placeholder' => '14',
					'autoload' => false,
				),

				array(
					'type' 	=> 'sectionend',
					'id' 	=> 'automatewoo_general'
				),

			);
		}

		return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings, $current_section );
	}


}


return new AW_Settings_Page();
