<?php
/**
 * @var $log AW_Model_Log
 */

$notes = $log->get_meta('notes');

$data_items = array();

$possible_data_items = array(
	'order_id',
	'guest_email',
	'user_id',
	'product_id',
	'category_id',
	'tag_id',
	'wishlist_id',
	'cart_id',
	'order_item_id',
	'order_note_id',
	'subscription_id',
	'comment_id'
);


foreach ( $possible_data_items as $item_key )
{
	if ( ! $value = $log->get_meta($item_key) )
		continue;

	switch ( $item_key )
	{
		case 'order_id':
			$link = get_edit_post_link($value);
			$data_items[] = array(
				'title' => __('Order', 'automatewoo'),
				'value' => "<a href='$link'>#$value</a>"
			);
			break;

		case 'user_id':
			$link = get_edit_user_link($value);
			$user = get_user_by('id', $value);
			$value = $user->user_firstname .' '. $user->user_lastname;

			if ( ! $value )
			{
				$value = $user->user_email;
			}

			$data_items[] = array(
				'title' => __('User', 'automatewoo'),
				'value' => "<a href='$link'>$value</a>"
			);
			break;

		case 'guest_email':
			$data_items[] = array(
				'title' => __('Guest', 'automatewoo'),
				'value' => "<a href='mailto:$value'>$value</a>"
			);
			break;

		case 'product_id':
			$link = get_edit_post_link($value);
			$value = get_the_title($value);
			$data_items[] = array(
				'title' => __('Product', 'automatewoo'),
				'value' => "<a href='$link'>$value</a>"
			);
			break;

		case 'subscription_id':
			$link = get_edit_post_link($value);
			$data_items[] = array(
				'title' => __('Subscription', 'automatewoo'),
				'value' => "<a href='$link'>#$value</a>"
			);
			break;

		default:
			$data_items[] = array(
				'title' => ucwords(str_replace('_', ' ', $item_key)),
				'value' => $value
			);
			break;
	}
}

?>

	<div class="automatewoo-modal-header">
		<h1><?php printf(__( "Log #%s", 'automatewoo' ), $log->id ) ?></h1>
	</div>

	<div class="automatewoo-modal-body">
		<div class="automatewoo-modal-body-inner">

			<ul>
				<li><strong><?php _e('Workflow', 'automatewoo') ?>:</strong> <a href="<?php echo get_edit_post_link( $log->workflow_id ) ?>"><?php echo get_the_title( $log->workflow_id ) ?></a></li>
				<li><strong><?php _e('Time', 'automatewoo') ?>:</strong> <?php echo aw_display_time( $log->date ) ?></li>

				<?php foreach ( $data_items as $item ): ?>
					<li><strong><?php echo $item['title'] ?>:</strong> <?php echo $item['value'] ?></li>
				<?php endforeach; ?>

				<li><strong><?php _e('Tracking enabled', 'automatewoo') ?>:</strong> <?php echo ( $log->tracking_enabled ? __('Yes','automatewoo') : __('No','automatewoo') ) ?></li>
				<li><strong><?php _e('Conversion tracking enabled', 'automatewoo') ?>:</strong> <?php echo ( $log->conversion_tracking_enabled ? __('Yes','automatewoo') : __('No','automatewoo') ) ?></li>

				<?php if ( $log->tracking_enabled ): ?>
					<li><strong><?php _e('Opened', 'automatewoo') ?>:</strong> <?php echo ( $log->has_open_recorded() ? aw_display_time($log->get_date_opened()) : __('No','automatewoo') ) ?></li>
					<li><strong><?php _e('Clicked', 'automatewoo') ?>:</strong> <?php echo ( $log->has_click_recorded() ? aw_display_time($log->get_date_clicked()) : __('No','automatewoo') ) ?></li>
				<?php endif; ?>

			</ul>

			<?php if ( $notes ): ?>
				<hr>

				<strong><?php _e( "Log notes:", 'automatewoo' ) ?></strong><br>
				<?php foreach ($notes as $note ): ?>
					<p><?php echo $note; ?></p>
				<?php endforeach; ?>

			<?php endif; ?>

		</div>
	</div>
