<?php
/**
 * @class       AW_Action
 * @package     AutomateWoo/Abstracts
 * @since       1.0.0
 */

abstract class AW_Action
{
	/** @var string */
	public $title;

	/** @var string */
	public $name;

	/** @var string */
	public $description;

	/** @var string */
	public $group = 'Other';

	/** @var array */
	public $fields = array();

	/** @var array */
	public $options;

	/** @var AW_Model_Workflow */
	public $workflow;

	/** @var bool */
	public $can_be_previewed = false;

	/** @var bool */
	protected $_fields_loaded = false;


	/**
	 * Fields only need to be loaded when editing a workflow
	 *
	 * @return mixed
	 */
	abstract public function load_fields();

	/**
	 * @return mixed
	 */
	abstract public function run();


	/**
	 * Construct
	 */
	public function __construct()
	{
		add_action( 'automatewoo_init_actions', array( $this, 'init' ) );
	}


	/**
	 * Construct
	 */
	public function init()
	{
		// Register the class
		AutomateWoo()->registered_actions[$this->name] = $this;
	}


	/**
	 * @param $option object
	 */
	public function add_field( $option )
	{
		$option->set_name_base( 'aw_workflow_data[actions]' );
		$this->fields[ $option->get_name() ] = $option;
	}


	/**
	 * @param $name
	 *
	 * @return mixed
	 */
	public function get_field( $name )
	{
		if ( ! $this->_fields_loaded )
		{
			$this->load_fields();
			$this->_fields_loaded = true;
		}

		if ( ! isset( $this->fields[$name] ) )
			return false;

		return $this->fields[$name];
	}


	/**
	 * @return mixed
	 */
	public function get_fields()
	{
		if ( ! $this->_fields_loaded )
		{
			$this->load_fields();
			$this->_fields_loaded = true;
		}

		return $this->fields;
	}


	/**
	 * @return string
	 */
	public function get_name()
	{
		return $this->name;
	}


	/**
	 * @param bool $prepend_group
	 * @return string
	 */
	public function get_title( $prepend_group = false )
	{
		if ( $prepend_group && $this->group != 'Other' )
		{
			return $this->group . ' - ' . $this->title;
		}

		return $this->title;
	}


	/**
	 * @return string|null
	 */
	public function get_description()
	{
		return $this->description;
	}

	/**
	 * @return string
	 */
	public function get_description_html()
	{
		if ( ! $this->get_description() )
			return '';

		return '<p class="aw-field-description">' . $this->get_description() .'</p>';
	}


	/**
	 * @param $options
	 */
	public function set_options($options)
	{
		$this->options = $options;
	}


	/**
	 * Will return all data if no $field is false
	 *
	 * @param bool $field
	 * @param bool $replace_vars
	 * @param bool $allow_html
	 *
	 * @return mixed
	 */
	public function get_option( $field, $replace_vars = false, $allow_html = false )
	{
		$field = wp_check_invalid_utf8( $field );
		$value = false;

		if ( isset( $this->options[$field] ) )
		{
			if ( $replace_vars )
			{
				$value = $this->workflow->variable_processor()->process_field( $this->options[$field], $allow_html );
			}
			else {
				$value = $this->options[$field];
			}
		}

		return apply_filters( 'automatewoo_action_option', $value, $field, $replace_vars, $this );
	}



	/**
	 *
	 */
	public function check_requirements() {}



	public function warning( $message )
	{
		if ( ! is_admin() ) return;
?>
		<script type="text/javascript">
			alert('ERROR: <?php echo $message ?>');
		</script>
<?php
	}


}