<?php
/**
 * $name and $title can be set for a specific trigger or they will fall back to their defaults
 *
 * @class       AW_Field
 * @package     AutomateWoo/Abstracts
 */

abstract class AW_Field
{
	/**
	 * @var string
	 */
	protected $default_title;

	/**
	 * @var string
	 */
	protected $default_name;

	/**
	 * @var string
	 */
	protected $title;

	/**
	 * @var string
	 */
	protected $name;

	/**
	 * @var string
	 */
	protected $type;

	/**
	 * Optional help info for the field
	 * @var string
	 */
	protected $description;

	/**
	 * @var string trigger or action
	 */
	protected $name_base;

	/**
	 * @var bool
	 */
	protected $required = false;

	/**
	 * @var string
	 */
	protected $classes = '';

	/**
	 * @var array
	 */
	protected $extra_attrs = array();

	/**
	 * @var string
	 */
	protected $placeholder = '';




	/**
	 * @return mixed
	 */
	abstract public function render( $value );


	/**
	 * @param $name
	 */
	public function set_name( $name )
	{
		$this->name = $name;
	}

	/**
	 * @param $title
	 */
	public function set_title( $title )
	{
		$this->title = $title;
	}


	/**
	 * @return string
	 */
	public function get_title()
	{
		if ( $this->title )
			return $this->title;

		return __( $this->default_title, 'automatewoo');
	}


	/**
	 * @return string
	 */
	public function get_name()
	{
		if ( $this->name )
			return $this->name;

		return $this->default_name;
	}


	/**
	 * @return string
	 */
	public function get_type()
	{
		return $this->type;
	}


	/**
	 * @param $description
	 */
	public function set_description( $description )
	{
		$this->description = $description;
	}


	/**
	 * @return string
	 */
	public function get_description()
	{
		return $this->description;
	}


	/**
	 * @param $placeholder string
	 */
	public function set_placeholder( $placeholder )
	{
		$this->placeholder = $placeholder;
	}


	/**
	 * @return string
	 */
	public function get_placeholder()
	{
		return $this->placeholder;
	}


	/**
	 * @param $classes string
	 */
	public function set_classes( $classes )
	{
		$this->classes = $classes;
	}


	/**
	 * @return string
	 */
	public function get_classes()
	{
		return $this->classes;
	}


	/**
	 * @param $name
	 * @param $value
	 */
	public function add_extra_attr( $name, $value )
	{
		$this->extra_attrs[$name] = $value;
	}


	/**
	 * @return string in HTML attribute format
	 */
	public function get_extra_attrs()
	{
		$string = '';

		foreach ( $this->extra_attrs as $name => $value )
		{
			$string .= $name . '="' . $value . '" ';
		}

		return $string;
	}


	/**
	 * @param bool $required
	 */
	public function set_required( $required = true )
	{
		$this->required = $required;
	}


	/**
	 * @return bool
	 */
	public function get_required()
	{
		return $this->required;
	}


	/**
	 * @param $name_base
	 */
	public function set_name_base( $name_base )
	{
		$this->name_base = $name_base;
	}


	/**
	 * @return bool
	 */
	public function get_name_base()
	{
		return $this->name_base;
	}

	/**
	 * @return string
	 */
	public function get_full_name()
	{
		return ( $this->get_name_base() ? $this->get_name_base() . '[' . $this->get_name() . ']' : $this->get_name() );
	}

}
