<?php
/**
 *
 *
 * @class       AW_Action_Active_Campaign_Add_Tag
 * @package     AutomateWoo/Actions
 * @since       2.0.0
 */

class AW_Action_Active_Campaign_Add_Tag extends AW_Action
{
	public $name = 'active_campaign_add_tag';

	public $group = 'ActiveCampaign';

	public $required_data_items = array(
		'user',
	);


	public function init()
	{
		$this->title = __('Add Tag To User', 'automatewoo');
		parent::init();
	}


	public function check_requirements()
	{
		if ( ! function_exists('curl_init') )
		{
			$this->warning( __('Server is missing CURL extension required to use the ActiveCampaign API.', 'automatewoo' ) );
		}
	}


	public function load_fields()
	{
		$tag = new AW_Field_Text_Input();
		$tag->set_title(__( 'Tag', 'automatewoo' ));
		$tag->set_name('tag');
		$tag->set_required(true);

		$create_user = new AW_Field_Checkbox();
		$create_user->set_title(__( "Create Contact If Missing", 'automatewoo' ));
		$create_user->set_name('create_missing_contact');

		$this->add_field($tag);
		$this->add_field($create_user);
	}


	/**
	 * @return void
	 */
	public function run()
	{
		if ( ! $user = $this->workflow->get_data_item('user') )
			return;

		$tag = $this->get_option('tag');
		$create_missing_contact = $this->get_option('create_missing_contact');

		if ( empty( $tag ) ) return;

		$ac = AutomateWoo()->integrations->get_active_campaign_wrapper();

		if  ( $create_missing_contact )
		{
			$contact_view = $ac->api('contact/view/email', array(
				'email' => $user->user_email
			));

			if ( ! $contact_view->success )
			{
				// insert the user and add the tag in the 1 request
				$contact_data = array(
					"email"              => $user->user_email,
					"first_name"         => $user->first_name,
					"last_name"          => $user->last_name,
					'tags' => $tag
				);

				$contact_sync = $ac->api("contact/sync", $contact_data);

				return;
			}
		}


		$data = array(
			'email' => $user->user_email,
			'tags' => $tag
		);

		$ac->api("contact/tag/add", $data);


	}

}
