<?php
/**
 * MailChimp Subscribe Action
 *
 * @class       AW_Action_MailChimp_Subscribe
 * @package     AutomateWoo/Actions
 * @since       2.0.3
 */

class AW_Action_MailChimp_Subscribe extends AW_Action
{
	public $name = 'mailchimp_subscribe';

	public $group = 'MailChimp';

	public $required_data_items = array(
		'user',
	);


	public function init()
	{
		$this->title = __('Add User to List', 'automatewoo');

		// Registers the actions
		parent::init();
	}


	public function load_fields()
	{
		$list_select = new AW_Field_Select();
		$list_select->set_title(__( 'List', 'automatewoo' ));
		$list_select->set_name('list');
		$list_select->set_options( AutomateWoo()->integrations->mailchimp()->get_lists() );
		$list_select->set_required();

		$double_optin = new AW_Field_Checkbox();
		$double_optin->set_name('double_optin');
		$double_optin->set_title( __( 'Double Optin', 'automatewoo' ) );
		$double_optin->set_description( __( 'Users will receive an email asking them to confirm their subscription.', 'automatewoo' ) );

		$this->add_field($list_select);
		$this->add_field($double_optin);
	}


	/**
	 * @return void
	 */
	public function run()
	{
		if ( ! $user = $this->workflow->get_data_item('user') )
			return;

		$list_id = $this->get_option('list');

		if ( ! $list_id )
			return;

		$subscriber_hash = md5(strtolower($user->user_email));
		$member_status = $this->get_option('double_optin') ? 'pending' : 'subscribed';

		AutomateWoo()->integrations->mailchimp()->request( 'PUT', "/lists/$list_id/members/$subscriber_hash", array(
			'email_address' => $user->user_email,
			'status' => $member_status,
			'merge_fields' => array(
				'FNAME' => $user->first_name,
				'LNAME' => $user->last_name
			),
		));
	}

}
