<?php
/**
 * MailChimp Unsubscribe Action
 *
 * @class       AW_Action_MailChimp_Unsubscribe
 * @package     AutomateWoo/Actions
 * @since       2.0.3
 */

class AW_Action_MailChimp_Unsubscribe extends AW_Action
{
	public $name = 'mailchimp_unsubscribe';

	public $group = 'MailChimp';

	public $required_data_items = array(
		'user',
	);


	public function init()
	{
		$this->title = __('Remove User From List', 'automatewoo');

		// Registers the actions
		parent::init();
	}


	public function load_fields()
	{
		$list_select = new AW_Field_Select();
		$list_select->set_title(__( 'List', 'automatewoo' ));
		$list_select->set_name('list');
		$list_select->set_options( AutomateWoo()->integrations->mailchimp()->get_lists() );
		$list_select->set_required();

		$unsubscribe_only = new AW_Field_Checkbox();
		$unsubscribe_only->set_name('unsubscribe_only');
		$unsubscribe_only->set_title( __( 'Unsubscribe Only', 'automatewoo' ) );
		$unsubscribe_only->set_description( __( 'If checked the user will be unsubscribed instead of deleted.', 'automatewoo' ) );

		$this->add_field($list_select);
		$this->add_field($unsubscribe_only);
	}


	/**
	 * @return void
	 */
	public function run()
	{
		if ( ! $user = $this->workflow->get_data_item('user') )
			return;

		$list_id = $this->get_option('list');
		$subscriber = md5(strtolower($user->user_email));

		if ( ! $list_id )
			return;

		if ( $this->get_option('unsubscribe_only') )
		{
			AutomateWoo()->integrations->mailchimp()->request( 'PATCH', "/lists/$list_id/members/$subscriber", array(
				'status' => 'unsubscribed',
			));
		}
		else
		{
			AutomateWoo()->integrations->mailchimp()->request( 'DELETE', "/lists/$list_id/members/$subscriber" );
		}
	}

}
