<?php
/**
 * Send SMS (Twilio) Action
 *
 * @class       AW_Action_Send_SMS_Twilio
 * @package     AutomateWoo/Actions
 * @since       1.1.9
 */

class AW_Action_Send_SMS_Twilio extends AW_Action
{
	public $name = 'send_sms_twilio';

	public $group = 'SMS';

	public $required_data_items = array();


	public function init()
	{
		$this->title = __('Send SMS (Twilio)', 'automatewoo');
		parent::init();
	}


	function load_fields()
	{
		$sms_recipient = new AW_Field_Text_Input();
		$sms_recipient->set_title( __( 'SMS Recipient', 'automatewoo'  ) );
		$sms_recipient->set_name('sms_recipient');

		$sms_body = new AW_Field_Text_Area();
		$sms_body->set_rows(2);
		$sms_body->set_title( __( 'SMS Body', 'automatewoo'  ) );
		$sms_body->set_name('sms_body');

		$this->add_field($sms_recipient);
		$this->add_field($sms_body);
	}


	/**
	 * @return mixed|void
	 */
	public function run()
	{
		if ( ! $user = $this->workflow->get_data_item('user') )
			return false;

		$sms_recipient = $this->get_option('sms_recipient', true );
		$sms_body = $this->get_option('sms_body', true );

		if ( empty( $sms_recipient ) ) return false;


		if ( $this->workflow->is_ga_tracking_enabled() )
		{
			$replacer = new AW_Replace_Helper( $sms_body, array( $this->workflow, 'append_ga_tracking_to_url' ), 'text_urls' );
			$sms_body = $replacer->process();
		}


		$sms_recipient = aw_parse_phone_number( $sms_recipient );

		AutomateWoo()->integrations->load_twilio();

		$from = AutomateWoo()->options->twilio_from;
		$sid = AutomateWoo()->options->twilio_auth_id;
		$token = AutomateWoo()->options->twilio_auth_token;

		$client = new Services_Twilio($sid, $token);

		try
		{
			$message = $client->account->messages->sendMessage( $from, $sms_recipient, $sms_body );
		}
		catch(Exception $e)
		{
			//var_dump( $e->getMessage() );
		}
	}

}
