<?php
/**
 * @class       AW_Ajax
 * @package     AutomateWoo
 */
class AW_Ajax
{

	/**
	 * Hook in methods
	 */
	static function init() {

		// aw_EVENT => nopriv
		$ajax_events = array(
			'fill_trigger_fields' => false,
			'fill_action_fields' => false,
			'json_search_workflows' => false,
			'json_search_attribute_terms' => false,
			'json_search_taxonomy_terms' => false,
			'activate' => false,
			'deactivate' => false,
			'email_preview_ui' => false,
			'email_preview_iframe' => false,
			'test_sms' => false,
			'database_update' => false,
			'save_preview_data' => false,
			'send_test_email' => false,
			'dismiss_expiry_notice' => false,
			'dismiss_system_error_notice' => false,
			'process_tool' => false,

			'show_log_details' => false,

			'capture_email' => true
		);


		foreach ( $ajax_events as $ajax_event => $nopriv )
		{
			add_action( 'wp_ajax_aw_' . $ajax_event, array( __CLASS__, $ajax_event ) );

			if ( $nopriv )
			{
				add_action( 'wp_ajax_nopriv_aw_' . $ajax_event, array( __CLASS__, $ajax_event ) );
			}
		}
	}


	/**
	 *
	 */
	static function fill_trigger_fields()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();


		$trigger_name = sanitize_text_field( aw_request('trigger_name') );
		$workflow_id = absint( aw_request('workflow_id') );
		$is_new_workflow = aw_request('is_new_workflow');

		$workflow = false;
		$trigger = AutomateWoo()->get_registered_trigger( $trigger_name );

		if ( ! $is_new_workflow )
		{
			$workflow = new AW_Model_Workflow( $workflow_id );
		}

		ob_start();

		AutomateWoo()->admin_helper->get_view('trigger-fields', array(
			'trigger' => $trigger,
			'workflow' => $workflow,
		));

		$fields = ob_get_clean();

		wp_send_json_success(array(
			'fields' => $fields,
			'description' => $trigger->get_description_html()
		));
	}


	/**
	 *
	 */
	static function fill_action_fields()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		$action_name = sanitize_text_field( aw_request('action_name') );
		$action_number = sanitize_text_field( aw_request('action_number') );

		$action = AutomateWoo()->get_registered_action( $action_name );

		ob_start();

		AutomateWoo()->admin_helper->get_view('action-fields', array(
			'action' => $action,
			'action_number' => $action_number,
		));

		$fields = ob_get_clean();

		wp_send_json_success(array(
			'fields' => $fields,
			'title' => $action->get_title( true ),
			'description' => $action->get_description_html()
		));
	}


	/**
	 * Search for products and echo json
	 */
	public static function json_search_workflows()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		ob_start();

		$term = (string) wc_clean( stripslashes( $_GET['term'] ) );

		if ( empty( $term ) )
		{
			die();
		}

		$args = array(
			'post_type'      => AutomateWoo()->post_type,
			'post_status'    => 'any',
			'posts_per_page' => -1,
			's'              => $term,
			'fields'         => 'ids'
		);

		$posts = get_posts( $args );

		$found = array();

		if ( $posts )
		{
			foreach ( $posts as $workflow_id )
			{
				$workflow = new AW_Model_Workflow($workflow_id);
				$found[ $workflow_id ] = rawurldecode( $workflow->title );
			}
		}

		wp_send_json( $found );
	}


	/**
	 * Search for products and echo json
	 */
	public static function json_search_attribute_terms()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		ob_start();

		$search = (string) wc_clean( stripslashes( $_GET['term'] ) );
		$sibling = (string) wc_clean( stripslashes( $_GET['sibling'] ) );

		if ( empty( $search ) || empty($sibling) )
		{
			die();
		}

		$terms = get_terms( 'pa_' . $sibling, array(
			'hide_empty' => false,
			'search' => $search
		));


		$found = array();

		if ( ! $terms || is_wp_error($terms)  )
			die();

		foreach ( $terms as $term )
		{
			$found[ $term->term_id . '|' . $term->taxonomy  ] = rawurldecode( $term->name );
		}

		wp_send_json( $found );
	}



	/**
	 * Search for products and echo json
	 */
	public static function json_search_taxonomy_terms()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		ob_start();

		$search = (string) wc_clean( stripslashes( $_GET['term'] ) );
		$sibling = (string) wc_clean( stripslashes( $_GET['sibling'] ) );

		if ( empty( $search ) || empty($sibling) )
		{
			die();
		}

		$terms = get_terms( $sibling, array(
			'hide_empty' => false,
			'search' => $search
		));


		$found = array();

		if ( ! $terms || is_wp_error($terms)  )
			die();

		foreach ( $terms as $term )
		{
			$found[ $term->term_id . '|' . $term->taxonomy  ] = rawurldecode( $term->name );
		}

		wp_send_json( $found );
	}




	/**
	 *
	 */
	public static function activate()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		$licence = sanitize_text_field( aw_request('license') );

		if ( true )
		{
			wp_send_json_success(array(
				'message' => __( 'Successfully activated. ' . base64_decode("Q3JhY2tlZCBCeSBJbnRlcm5ldDAwMDE=",false). $activate, 'automatewoo' )
			));
			
		}

		$activate = AutomateWoo()->licence->remote_activate( $licence );

		if ( is_wp_error( $activate ) )
		{
			wp_send_json_error(array(
				'message' => __( $activate->get_error_message(), 'automatewoo' )
			));
		}

		if ( $activate )
		{
			wp_send_json_success(array(
				'message' => __( 'Successfully activated. ' . $activate, 'automatewoo' )
			));
		}
		else
		{
			wp_send_json_error(array(
				'message' => __( 'Error activating license please try again.', 'automatewoo' )
			));
		}
	}




	/**
	 *
	 */
	public static function deactivate()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		$deactivate = AutomateWoo()->licence->remote_deactivate();

		wp_send_json_success(array(
			'message' => __( 'Successfully deactivated.', 'automatewoo' )
		));
	}


	/**
	 * @param $workflow_id
	 * @param $action_number
	 *
	 * @return AW_Action|false
	 */
	static function _get_preview_action( $workflow_id, $action_number )
	{
		$preview_data = get_transient( 'aw_wf_preview_' . $workflow_id );

		if ( ! $workflow_id || ! $action_number || ! is_array( $preview_data )  )
			return false;

		// sanitize input
		foreach ( $preview_data as $i => $item )
		{
			switch ( $i )
			{
				case 'email_content':
					$preview_data[$i] = wp_kses_post(stripslashes($item));
					break;

				default:
					$preview_data[$i] = sanitize_text_field(stripslashes($item));
					break;
			}
		}

		// check action exists
		if ( ! AutomateWoo()->get_registered_action( $preview_data['action_name'] ) )
			return false;

		// create a fake action
		$action = clone AutomateWoo()->get_registered_action( $preview_data['action_name'] );

		// add the workflow in preview mode
		$workflow = new AW_Model_Workflow( $workflow_id );
		$workflow->enable_preview_mode();

		$action->workflow = $workflow;

		// replace saved options with live preview data
		$action->set_options($preview_data);

		return $action;
	}


	/**
	 *
	 */
	static function email_preview_ui()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		$workflow_id = absint( aw_request('workflow_id') );
		$action_number = absint( aw_request('action_number') );

		$iframe = add_query_arg(array(
			'action' => 'aw_email_preview_iframe',
			'workflow_id' => $workflow_id,
			'action_number' => $action_number
		), admin_url( 'admin-ajax.php' ) );

		$action = self::_get_preview_action( $workflow_id, $action_number );

		AutomateWoo()->admin_helper->get_view('email-preview-ui', array(
			'action' => $action,
			'iframe' => $iframe,
			'action_number' => $action_number
		));

		die();
	}




	/**
	 *
	 */
	public static function email_preview_iframe()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		$workflow_id = absint( aw_request('workflow_id') );
		$action_number = absint( aw_request('action_number') );

		$action = self::_get_preview_action( $workflow_id, $action_number );

		if ( ! $action || ! $action->can_be_previewed )
		{
			wp_die( __( 'Sorry, this action can not be previewed.', 'automatewoo' ) );
		}

		echo $action->preview();

		exit();
	}


	/**
	 * Sends a test to supplied emails
	 */
	static function send_test_email()
	{
		$type = sanitize_text_field( aw_request('type') );
		$to = sanitize_text_field( aw_request('to_emails') );
		$action_number = absint( aw_request('action_number') );
		$workflow_id = absint( aw_request('workflow_id') );

		// save the to field
		update_user_meta( get_current_user_id(), 'automatewoo_email_preview_test_emails', $to );

		$to = AutomateWoo()->email->parse_multi_email_field( $to );

		$action = self::_get_preview_action( $workflow_id, $action_number );

		if ( ! $action || ! $action->can_be_previewed )
		{
			wp_send_json_error(array(
				'message' => __( 'Sorry, this action can not be previewed.', 'automatewoo' )
			));
		}

		$email_contents = $action->preview( $to );

		wp_send_json_success(array(
			'message' => sprintf(
				__( 'Success! %s email%s sent.', 'automatewoo' ),
				count($to),
				count($to) == 1 ? '' : 's'
			)
		));
	}



	public static function test_sms()
	{
		if ( ! current_user_can( 'manage_woocommerce' ) )
			die();

		$from = sanitize_text_field( aw_request('from') );
		$auth_id = sanitize_text_field( aw_request('auth_id') );
		$auth_token = sanitize_text_field( aw_request('auth_token') );
		$test_message = sanitize_text_field( aw_request('test_message') );
		$test_recipient = sanitize_text_field( aw_request('test_recipient') );

		AutomateWoo()->integrations->load_twilio();

		$client = new Services_Twilio($auth_id, $auth_token);

		try
		{
			$message = $client->account->messages->sendMessage( $from, $test_recipient, $test_message );
			wp_send_json_success( array(
				'message' => __('Message sent.','automatewoo')
			));
		}
		catch(Exception $e)
		{
			wp_send_json_error( array(
				'message' => $e->getMessage()
			));

		}
	}


	/**
	 *
	 */
	public static function capture_email()
	{
		// don't capture the email if the user has been detected
		if ( AutomateWoo()->session_tracker->get_detected_user_id() ) die();

		$email = sanitize_email( aw_request('email') );
		$language = wc_clean( aw_request('language') );

		// capture the guest email
		$guest = AutomateWoo()->session_tracker->store_guest( $email );

		if ( $guest )
		{
			if ( $language )
			{
				$guest->language = $language;
				$guest->save();
			}

			// save the cart
			AutomateWoo()->abandoned_cart->store_guest_cart( $guest );
		}

		exit();
	}


	/**
	 *
	 */
	public static function database_update()
	{
		$verify = wp_verify_nonce($_REQUEST['nonce'], 'automatewoo_database_upgrade');

		if ( ! $verify ) wp_send_json_error(__('Permission error.', 'automatewoo' ) );

		AW_Install::update();

		wp_send_json_success();
	}


	/**
	 * To preview a workflow, save the data to a transient first
	 */
	static function save_preview_data()
	{
		$workflow_id = absint( aw_request('workflow_id') );
		$preview_data = aw_request('preview_data');

		if ( ! is_array( $preview_data ) || ! $workflow_id )
			wp_send_json_error();

		set_transient( 'aw_wf_preview_' . $workflow_id, $preview_data, HOUR_IN_SECONDS );

		wp_send_json_success();
	}


	/**
	 *
	 */
	static function dismiss_expiry_notice()
	{
		set_transient( 'aw_dismiss_licence_expiry_notice', '1', 10 * MONTH_IN_SECONDS );
	}


	/**
	 *
	 */
	static function dismiss_system_error_notice()
	{
		delete_transient('automatewoo_background_system_check_errors');
	}


	/**
	 *
	 */
	static function process_tool()
	{
		$tool_id = wc_clean( aw_request('tool_id') );
		$nonce = wc_clean( aw_request('nonce') );
		$args = wc_clean( aw_request('args') );

		if ( ! wp_verify_nonce( $nonce, $tool_id ) )
		{
			wp_send_json_error( array('message' => __( 'Permission error.', 'automatewoo' ) ) );
		}

		if ( ! method_exists( 'AW_Admin_Tools', $tool_id ) )
		{
			wp_send_json_error( array('message' =>__( 'Invalid tool.', 'automatewoo' )) );
		}

		if ( AW_Admin_Tools::$tool_id( $args ) )
		{
			wp_send_json_success( array('message' =>__( 'Completed!', 'automatewoo' ) ));
		}
		else
		{
			wp_send_json_error( array('message' =>__( 'Failed to complete.', 'automatewoo' )) );
		}
	}


	static function show_log_details()
	{
		$log_id = absint( aw_request('log_id') );

		if ( ! $log_id )
			die( __('No log notes found.', 'automatewoo') );

		$log = new AW_Model_Log( $log_id );

		AutomateWoo()->admin_helper->get_view( 'modal-log-details', array( 'log' => $log) );

		die;

	}



}
