<?php

/**
 * Class AW_Integrations
 *
 * Integration helpers
 */
class AW_Integrations
{
	/** @var AW_Integration_Mailchimp */
	private $mailchimp;


	/**
	 * @return bool
	 */
	function is_wpml()
	{
		return class_exists('SitePress');
	}


	/**
	 * @return bool
	 */
	function is_woo_pos()
	{
		return class_exists('WC_POS');
	}


	/**
	 * @return bool
	 */
	public function subscriptions_enabled()
	{
		if ( ! class_exists( 'WC_Subscriptions' ) ) return false;
		if ( WC_Subscriptions::$version < '2.0.0' ) return false;
		return true;
	}



	public function load_twilio()
	{
		if ( ! function_exists( 'Services_Twilio_autoload' ) )
		{
			require_once AutomateWoo()->lib_path . '/twilio-php/Services/Twilio.php';
		}
	}


	public function load_campaignmonitor()
	{
		if ( ! class_exists( 'CS_REST_Subscribers' ) )
		{
			require_once AutomateWoo()->lib_path . '/campaignmonitor-createsend-php/csrest_subscribers.php';
		}
	}


	public function load_madmimi()
	{
		if ( ! class_exists( 'MadMimi' ) )
		{
			require_once AutomateWoo()->lib_path . '/madmimi-php/MadMimi.class.php';
		}
	}


	public function load_activecampaign()
	{
		if ( ! class_exists( 'ActiveCampaign' ) )
		{
			require_once AutomateWoo()->lib_path . '/activecampaign-api-php/includes/ActiveCampaign.class.php';
		}
	}


	/**
	 * @return AW_Integration_Mailchimp
	 */
	public function mailchimp()
	{
		if ( ! $this->mailchimp )
		{
			$this->mailchimp = new AW_Integration_Mailchimp( AutomateWoo()->options->mailchimp_api_key );
		}

		return $this->mailchimp;
	}


	/**
	 * @return ActiveCampaign
	 */
	public function get_active_campaign_wrapper()
	{
		AutomateWoo()->integrations->load_activecampaign();

		$api_url = AutomateWoo()->options->active_campaign_api_url;
		$api_key = AutomateWoo()->options->active_campaign_api_key;

		return new ActiveCampaign( $api_url, $api_key );
	}



	/**
	 * @return array
	 */
	public function get_active_campaign_lists()
	{
		$cache = get_transient('automatewoo_active_campaign_lists');

		if ( $cache ) return $cache;


		$ac = AutomateWoo()->integrations->get_active_campaign_wrapper();
		$lists = $ac->api( 'list/list', array( 'ids' => 'all' ) );
		$clean_lists = array();

		foreach ( $lists as $list )
		{
			if ( is_object($list) )
			{
				$clean_lists[$list->id] = $list->name;
			}
		}

		set_transient('automatewoo_active_campaign_lists', $clean_lists, 10 * MINUTE_IN_SECONDS );

		return $clean_lists;
	}

}


