<?php
/**
 * @class       AW_Model_Log
 * @package     AutomateWoo/Models
 * @since       2.1.0
 *
 * @property int $workflow_id
 * @property string $date UTC
 * @property bool $tracking_enabled
 * @property bool $conversion_tracking_enabled
 */

class AW_Model_Log extends AW_Model
{
	/**
	 * @var AW_Model_Workflow
	 */
	private $workflow;


	public $meta_id_column = 'log_id';


	/**
	 * @param bool|int $id
	 */
	public function __construct( $id = false )
	{
		$this->table_name = AutomateWoo()->table_name_logs;
		$this->meta_table_name = AutomateWoo()->table_name_log_meta;

		if ( $id ) $this->get_by( 'id', $id );
	}


	/**
	 * @param $url
	 */
	public function record_click( $url )
	{
		if ( ! $tracking = $this->get_meta('tracking_data') )
			$tracking = array();

		$tracking[] = array(
			'type' => 'click',
			'url' => $url,
			'date' => current_time( 'mysql', true )
		);

		$this->update_meta( 'tracking_data', $tracking );

		// clicking requires an open so record one just in case images were blocked
		if ( ! $this->has_open_recorded() )
		{
			$this->record_open();
		}
	}


	/**
	 * Only records an open once i.e. unique opens
	 */
	public function record_open()
	{
		if ( ! $tracking = $this->get_meta('tracking_data') )
			$tracking = array();

		foreach( $tracking as $item )
			if ( $item['type'] == 'open')
				return;

		$tracking[] = array(
			'type' => 'open',
			'date' => current_time( 'mysql', true )
		);

		$this->update_meta( 'tracking_data', $tracking );
	}


	/**
	 * @return bool
	 */
	public function has_open_recorded()
	{
		$tracking = $this->get_meta('tracking_data');

		if ( is_array( $tracking ) ) foreach( $tracking as $item )
		{
			if ( $item['type'] == 'open') return true;
		}

		return false;
	}


	/**
	 * @return bool
	 */
	public function has_click_recorded()
	{
		$tracking = $this->get_meta('tracking_data');

		if ( is_array( $tracking ) ) foreach( $tracking as $item )
		{
			if ( $item['type'] == 'click') return true;
		}

		return false;
	}


	/**
	 * @return string|false
	 */
	public function get_date_opened()
	{
		$tracking = $this->get_meta('tracking_data');

		if ( is_array( $tracking ) ) foreach( $tracking as $item )
		{
			if ( $item['type'] == 'open') return $item['date'];
		}

		return false;
	}


	/**
	 * @return string|false
	 */
	public function get_date_clicked()
	{
		$tracking = $this->get_meta('tracking_data');

		if ( is_array( $tracking ) ) foreach( $tracking as $item )
		{
			if ( $item['type'] == 'click') return $item['date'];
		}

		return false;
	}



	/**
	 * @param $note string
	 */
	public function add_note( $note )
	{
		$notes = $this->get_meta( 'notes' );
		if ( ! is_array( $notes ) ) $notes = array();
		$notes[] = $note;
		$this->update_meta( 'notes', $notes );
	}


	/**
	 *
	 */
	public function delete()
	{
		// delete conversion records for the log
		$converted_orders = get_posts(array(
			'post_type' => 'shop_order',
			'post_status' => 'any',
			'posts_per_page' => -1,
			'fields' => 'ids',
			'meta_query' => array(
				array(
					'key' => '_aw_conversion_log',
					'value' => $this->id
				)
			)
		));

		if ( $converted_orders ) foreach ( $converted_orders as $order_id )
		{
			delete_post_meta( $order_id, '_aw_conversion' );
			delete_post_meta( $order_id, '_aw_conversion_log' );
		}

		parent::delete();
	}


	/**
	 * @return AW_Model_Workflow
	 */
	public function get_workflow()
	{
		if ( $this->workflow ) return $this->workflow;

		return $this->workflow = new AW_Model_Workflow( $this->workflow_id );
	}


}

