<?php

/**
 * WP option wrapper
 *
 * @class AW_Options
 * @version 2.0.2
 *
 * @property string $version
 *
 * @property bool $twilio_integration_enabled
 * @property string $twilio_from
 * @property string $twilio_auth_id
 * @property string $twilio_auth_token
 *
 * @property bool $mailchimp_integration_enabled
 * @property bool $mailchimp_api_key
 *
 * @property bool $active_campaign_integration_enabled
 * @property string $active_campaign_api_url
 * @property string $active_campaign_api_key
 *
 * @property string $url_shortener_enabled
 * @property string $google_api_key
 *
 * @property string $guest_email_capture_scope (checkout,all,none)
 *
 * @property int $queue_batch_size
 * @property int $conversion_window
 *
 * @property bool $enable_background_system_check
 */
class AW_Options
{

	/**
	 * @var string
	 */
	public $prefix = 'automatewoo_';


	/**
	 * @var array
	 */
	public $defaults = array(
		'twilio_integration_enabled' => false,
		'active_campaign_integration_enabled' => false,
		'mailchimp_integration_enabled' => false,
		'url_shortener_enabled' => false,
		'guest_email_capture_scope' => 'checkout',
		'queue_batch_size' => 25,
		'conversion_window' => 14,
		'enable_background_system_check' => true,
	);


	/**
	 * Magic method for option retrieval
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function __get( $key )
	{
		if ( $value = get_option( $this->prefix . $key ) )
			return $this->parse($value);

		// fallback to default
		if ( isset( $this->defaults[$key] ) )
			return $this->parse( $this->defaults[$key] );

		return false;
	}


	/**
	 * Convert yes / no strings to boolean
	 *
	 * @param $value
	 *
	 * @return mixed
	 */
	public function parse( $value )
	{
		if ( $value === 'yes' ) return true;
		if ( $value === 'no' ) return false;
		return $value;
	}
}

