<?php
/**
 * @class 		AW_Remote_Request
 * @since		2.3.1
 * @package		AutomateWoo
 */

class AW_Remote_Request
{
	/** @var array */
	public $http_success_codes = array( 200, 201, 202, 203, 204 );

	/**
	 * Response from wp_remote_request()
	 * @var array|WP_Error
	 */
	public $request;


	/**
	 * Passes to wp_remote_request()
	 *
	 * @param $url
	 * @param $args
	 */
	public function __construct( $url, $args )
	{
		$this->request = wp_remote_request( $url, $args );
	}


	/**
	 * @return bool
	 */
	public function is_failed()
	{
		return is_wp_error( $this->request );
	}


	/**
	 * @return string
	 */
	public function get_error_message()
	{
		if ( $this->is_failed() )
		{
			return $this->request->get_error_message();
		}
	}


	/**
	 * @return int
	 */
	public function get_response_code()
	{
		if ( $this->is_failed() )
			return 0;

		return $this->request['response']['code'];
	}


	/**
	 * @return string
	 */
	public function get_response_message()
	{
		if ( $this->is_failed() )
			return '';

		return $this->request['response']['message'];
	}


	/**
	 * @return array|false
	 */
	public function get_body()
	{
		if ( $this->is_failed() )
			return false;

		return json_decode( $this->request['body'], true );
	}


	/**
	 * @return string
	 */
	public function get_body_raw()
	{
		if ( $this->is_failed() )
			return '';

		return $this->request['body'];
	}


	/**
	 * @return bool
	 */
	public function is_http_success_code()
	{
		return in_array( $this->get_response_code(), $this->http_success_codes );
	}


	/**
	 * @return bool
	 */
	public function is_successful()
	{
	 return ! $this->is_failed() && $this->is_http_success_code();
	}


}