<?php
/***
 * @class       AW_Trigger_Order_Note_Added
 * @package     AutomateWoo/Triggers
 * @since       2.2
 */

class AW_Trigger_Order_Note_Added extends AW_Trigger
{
	public $name = 'order_note_added';

	public $group = 'Order';

	public $supplied_data_items = array( 'order', 'user', 'shop', 'order_note' );

	public function init()
	{
		$this->title = __('Order Note Added', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$contains = new AW_Field_Text_Input();
		$contains->set_name('note_contains');
		$contains->set_title( __( 'Note Contains Text', 'automatewoo'  ) );
		$contains->set_description( __( 'Only trigger if the order note contains the following text.', 'automatewoo'  ) );

		$user_type = new AW_Field_User_Type( true, false );
		$user_type->set_description( __( 'Only trigger when the review is made by a user of a certain type.', 'automatewoo'  ) );

		$this->add_field( $contains );
		$this->add_field( $user_type );
		$this->add_user_limit_field();
	}



	/**
	 * When could this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'wp_insert_comment', array( $this, 'catch_hooks' ), 20, 2 );
	}


	/**
	 * Catch any comments approved on creation
	 *
	 * @param $comment_id
	 * @param $comment WP_Comment
	 */
	public function catch_hooks( $comment_id, $comment )
	{
		if ( $comment->comment_type !== 'order_note' )
			return;

		$order = wc_get_order( $comment->comment_post_ID );
		$user = $order->get_user();

		// order placed by a guest
		if ( ! $user )
		{
			$user = new AW_Model_Order_Guest( $order );
		}

		$order_note = new AW_Model_Order_Note( $comment->comment_ID, $comment->comment_content, $order->id );

		$this->maybe_run(array(
			'user' => $user,
			'order' => $order,
			'order_note' => $order_note
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');
		$order_note = $workflow->get_data_item('order_note');

		if ( ! $user || ! $order || ! $order_note )
			return false;

		$note_contains = $trigger->get_option('note_contains');

		if ( $note_contains )
		{
			if ( ! stristr( $order_note->note, $note_contains ) )
				return false;
		}

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// validate user limit
		if ( ! $this->validate_limit_per_user( $workflow, $trigger, $user ) )
			return false;

		return true;
	}


}
