<?php
/**
 * @class       AW_Trigger_Order_Status_Changes
 * @package     AutomateWoo/Triggers
 */

class AW_Trigger_Order_Status_Changes extends AW_Trigger_Abstract_Order_Status_Base
{

	public $name = 'order_status_changes';


	/**
	 * Construct
	 */
	public function init()
	{
		$this->title = __('Order Status Changes', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * When could this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'woocommerce_order_status_changed', array( $this, 'catch_hooks' ), 100, 3 );
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$from = new AW_Field_Order_Status();
		$from->set_title( __( 'Order Status Changes From', 'automatewoo'  ) );
		$from->set_name('order_status_from');

		$to = new AW_Field_Order_Status();
		$to->set_title( __( 'Order Status Changes To', 'automatewoo'  ) );
		$to->set_name('order_status_to');

		$this->add_field($from);
		$this->add_field($to);

		parent::load_fields();
	}



	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		/**
		 * @var WP_User $user
		 * @var WC_Order $order
		 */
		$trigger = $workflow->get_trigger();
		$order = $workflow->get_data_item('order');

		if ( ! $order ) return false;

		// get options
		$order_status_from = $trigger->get_option('order_status_from');
		$order_status_to = $trigger->get_option('order_status_to');

		if ( ! $this->validate_status_field( $order_status_from, $order->_aw_old_status ) )
			return false;

		if ( ! $this->validate_status_field( $order_status_to, $order->get_status() ) )
			return false;

		if ( ! parent::validate_workflow( $workflow ) )
			return false;

		return true;
	}


	/**
	 * Ensures 'to' status has not changed while sitting in queue
	 *
	 * @param $workflow
	 *
	 * @return bool
	 */
	public function validate_before_queued_event( $workflow )
	{
		// check parent
		if ( ! parent::validate_before_queued_event( $workflow ) )
			return false;

		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');

		if ( ! $user || ! $order )
			return false;

		// Option to validate order status
		if ( $trigger->get_option('validate_order_status_before_queued_run') )
		{
			$order_status_to = $trigger->get_option('order_status_to');

			if ( ! $this->validate_status_field( $order_status_to, $order->get_status() ) )
				return false;
		}

		return true;
	}


}
