<?php
/**
 *
 *
 * @class       AW_Trigger_Subscription_Payment_Failed
 * @package     AutomateWoo/Triggers
 * @since       2.1.0
 */

class AW_Trigger_Subscription_Payment_Failed extends AW_Trigger_Abstract_Subscriptions
{
	public $name = 'subscription_payment_failed';

	/**
	 * Construct
	 */
	public function init()
	{
		$this->title = __('Subscription Payment Failed', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$product = new AW_Field_Subscription_Products();
		$product->set_description( __( 'Select which subscription products to trigger for. Leave blank to apply for all subscription products.', 'automatewoo'  ) );

		$skip_first = new AW_Field_Checkbox();
		$skip_first->set_name('skip_first');
		$skip_first->set_title( __( 'Skip First Payment', 'automatewoo'  ) );

		$this->add_field($product);
		$this->add_field($skip_first);
		$this->add_user_tags_field();
	}



	/**
	 * When might this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'woocommerce_subscription_payment_failed', array( $this, 'catch_hooks' ), 20, 1 );
	}


	/**
	 * Route hooks through here
	 *
	 * @param $subscription WC_Subscription
	 */
	public function catch_hooks( $subscription )
	{
		$this->maybe_run(array(
			'subscription' => $subscription,
			'user' => $subscription->get_user()
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');

		/** @var $subscription WC_Subscription */
		$subscription = $workflow->get_data_item('subscription');

		if ( ! $user || ! $subscription )
			return false;

		// options
		$skip_first = $trigger->get_option('skip_first');


		if ( $skip_first )
		{
			if ( $subscription->get_completed_payment_count() <= 2 ) return false;
		}

		if ( ! $this->validate_subscription_products_field( $subscription, $trigger ) )
			return false;

		if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
			return false;


		return true;
	}

}
