<?php
/**
 * @class       AW_Trigger_User_New_Account
 * @package     AutomateWoo/Triggers
 * @since       1.0.0
 */

class AW_Trigger_User_New_Account extends AW_Trigger
{
	public $name = 'user_new_account';

	public $group = 'User';

	public $supplied_data_items = array(
		'user',
		'shop'
	);


	public function init()
	{
		$this->title = __('New User Account Created', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type( true, false );
		$user_type->set_description( __( 'Only trigger for users of a certain type.', 'automatewoo'  ) );

		$this->add_field( $user_type );
	}



	/**
	 * When might this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'user_register', array( $this, 'catch_hooks' ) );
	}


	/**
	 * Route hooks through here
	 * @param int $user_id
	 */
	public function catch_hooks( $user_id )
	{
		if ( get_user_meta( $user_id, '_aw_user_registered', true ) ) return;

		add_user_meta( $user_id, '_aw_user_registered', true );

		$user = get_user_by( 'id', $user_id );

		$this->maybe_run(array(
			'user' => $user
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');

		if ( ! $user )
			return false;

		// validate user type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		return true;
	}

}
