<?php
/**
 *
 *
 * @class       AW_Trigger_User_Order_Count_Reaches
 * @package     AutomateWoo/Triggers
 * @since       2.0.0
 */

class AW_Trigger_User_Order_Count_Reaches extends AW_Trigger
{

	public $title;

	public $name = 'users_order_count_reaches';

	public $group = 'User';

	public $supplied_data_items = array( 'user', 'shop' );


	/**
	 * Construct
	 */
	public function init()
	{
		$this->title = __('User Orders Count Reaches', 'automatewoo');
		$this->description = __("This trigger checks the user's order count each time an order is completed. Note that it does not work for guests.", 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$user_type = new AW_Field_User_Type( true, false );
		$user_type->set_description( __( 'Only trigger for users of a certain type.', 'automatewoo'  ) );

		$order_count = new AW_Field_Number_Input();
		$order_count->set_name( 'order_count' );
		$order_count->set_title( __( 'Order Count', 'automatewoo' ) );
		$order_count->set_required(true);

		$this->add_field( $order_count );
		$this->add_field( $user_type );
		$this->add_user_tags_field();
	}



	/**
	 * Must run on complete after WC has updated its records
	 */
	public function register_hooks()
	{
		add_action( 'woocommerce_order_status_completed', array( $this, 'catch_hooks' ), 100, 1 );
	}



	/**
	 * Route hooks through here
	 */
	public function catch_hooks( $order_id )
	{
		$order = wc_get_order( $order_id );
		$user = $order->get_user();

		if ( ! $user ) return; // doesn't work for guests

		$this->maybe_run(array(
			'order' => $order,
			'user' => $user
		));
	}


	/**
	 * @param $workflow AW_Model_Workflow
	 *
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();
		$user = $workflow->get_data_item('user');
		$order = $workflow->get_data_item('order');

		if ( ! $user || ! $order )
			return false;

		// Options
		$order_count = absint( $trigger->get_option('order_count') );

		// fail if no order count set
		if ( ! $order_count )
			return false;

		// Only do this once for each user (for each workflow)
		if ( ! $workflow->is_first_run_for_user( $user ) )
			return false;

		// Validate User type
		if ( ! $this->validate_user_type_field( $trigger, $user ) )
			return false;

		// validate user tags
		if ( ! $this->validate_user_tag_fields( $user, $trigger ) )
			return false;

		// Validate order count
		if ( absint( wc_get_customer_order_count( $user->ID ) ) <= $order_count )
			return false;

		return true;
	}

}
