<?php
/**
 * @class       AW_Trigger_Workflow_Times_Run_Reaches
 * @package     AutomateWoo/Triggers
 */

class AW_Trigger_Workflow_Times_Run_Reaches extends AW_Trigger
{
	public $name = 'workflow_times_run_reaches';

	public $group = 'Workflow';

	public $supplied_data_items = array( 'post', 'workflow', 'shop' );


	public function init()
	{
		$this->title = __('Workflow Times Run Reaches', 'automatewoo');

		// Registers the trigger
		parent::init();
	}


	/**
	 * Add options to the trigger
	 */
	public function load_fields()
	{
		$workflow_field = new AW_Field_Workflow();

		$times_run = new AW_Field_Number_Input();
		$times_run->set_name('times_run');
		$times_run->set_title(__('Times Run', 'automatewoo') );

		$this->add_field( $workflow_field );
		$this->add_field( $times_run );
	}



	/**
	 * When could this trigger run?
	 */
	public function register_hooks()
	{
		add_action( 'automatewoo_after_workflow_run', array( $this, 'catch_hooks' ) );
	}


	/**
	 * Route hooks through here
	 *
	 * @param $workflow AW_Model_Workflow
	 */
	public function catch_hooks( $workflow )
	{
		$this->maybe_run(array(
			'workflow' => $workflow,
			'post' => $workflow->post
		));
	}


	/**
	 * @return bool
	 */
	public function validate_workflow( $workflow )
	{
		$trigger = $workflow->get_trigger();

		/** @var $workflow_data_item AW_Model_Workflow */
		$workflow_data_item = $workflow->get_data_item('workflow');

		$selected_workflow_id = absint( $trigger->get_option('workflow') );
		$times_run = absint( $trigger->get_option('times_run') );

		if ( ! $workflow_data_item )
			return false;

		// match running workflow to selected workflow
		if ( $workflow_data_item->id != $selected_workflow_id )
			return false;

		if ( $workflow_data_item->get_times_run() !== $times_run )
			return false;

		return true;
	}


}
