<?php
/**
 * Updater
 *
 * @class       AW_Updater
 * @package     AutomateWoo
 * @since       1.0.0
 */

class AW_Updater
{

	/**
	 * Add actions
	 */
	static function init()
	{
		add_filter( 'plugins_api', array( __CLASS__, 'check_info' ), 10, 3 );
		add_filter( 'pre_set_site_transient_update_plugins', array( __CLASS__, 'inject_update' ) );
		add_action( 'in_plugin_update_message-' . AutomateWoo()->plugin_basename, array( __CLASS__, 'in_plugin_update_message' ) );
	}


	/**
	 * Check for updates
	 *
	 * @param $transient
	 *
	 * @return
	 */
	static function inject_update( $transient )
	{
		if ( empty( $transient->checked ) )
			return $transient;

		if ( ! $version = self::update_available() )
			return $transient;


		$obj = new stdClass();
		$obj->slug = AutomateWoo()->plugin_slug;
		$obj->plugin = AutomateWoo()->plugin_basename;
		$obj->new_version = $version;
		$obj->url = AutomateWoo()->website_url;

		$licence_type = AutomateWoo()->licence->get_type();

		if ( in_array( $licence_type, array( 'live', 'dev' ) ) )
		{
			$licence = AutomateWoo()->licence->get();

			$package_url = add_query_arg(array(
				'wc-api' => 'licences',
				'request' => 'app_download',
				'licence_key' => $licence['key'],
				'instance' => urlencode( $licence['url'] ),
				'app_id' => AutomateWoo()->plugin_slug
			), AutomateWoo()->website_url );

			$obj->package = $package_url;
		}
		else
		{
			$obj->upgrade_notice = '<div class="woothemes-updater-plugin-upgrade-notice">To enable updates please enter your license key on the WooCommerce &gt; Settings &gt; AutomateWoo page.</div>';
		}


		if ( $transient )
		{
			$transient->response[ AutomateWoo()->plugin_basename ] = $obj;
			return $transient;
		}
	}



	/**
	 * Check if there is a new version regardless of licence type.
	 *
	 * @return bool
	 */
	static function update_available()
	{
		$args = array();

		if ( AutomateWoo()->licence->get_type() == 'live' )
		{
			$licence = AutomateWoo()->licence->get();

			$args = array(
				'licence_key' => $licence['key'],
				'instance' => $licence['url'],
				'version' => AutomateWoo()->version
			);
		}

		$response = AutomateWoo()->licence->remote_get( 'app_version', $args );

		if ( $response );
		{
			if ( $response->success )
			{
				if ( version_compare( $response->version, AutomateWoo()->version, '<=' ) )
				{
					return false;
				}

				return $response->version;
			}
		}
	}



	/**
	 *
	 */
	static function in_plugin_update_message()
	{
		$m = false;

		$licence_type = AutomateWoo()->licence->get_type();

		if ( $licence_type == 'expired' )
		{
			$m = sprintf(
				__('<strong>Your licence has expired.</strong> <a href="%s" target="_blank">Renew your licence</a> to receive updates and support.', 'automatewoo'),
				AutomateWoo()->licence->get_renewal_url()
			);
		}
		elseif ( ! in_array( $licence_type, array( 'live', 'dev' ) ) )
		{
			$m = sprintf(
				__('To enable updates, please enter your licence key on the <a href="%s"><strong>AutomateWoo Settings</strong></a> page. If you don\'t have a license key, please see <a href="%s">details & pricing</a>', 'automatewoo'),
				admin_url('admin.php?page=wc-settings&tab=automatewoo&section=licence'),
				AutomateWoo()->website_url
			);
		}

		if ( ! $m ) return;

		echo '<br /><div>' . $m . '</div>';
	}


	/**
	 * @param $false
	 * @param $action
	 * @param $arg
	 *
	 * @return mixed
	 */
	static function check_info( $false, $action, $arg )
	{
		if ( isset($arg->slug) && $arg->slug === AutomateWoo()->plugin_slug )
		{
			if ( $response = AutomateWoo()->licence->remote_get( 'app_wp_plugin_info' ) )
			{
				if ( $response->success )
				{
					$info = $response->info;

					$info->sections = get_object_vars( $info->sections );
					return $info;
				}
			}

		}
		return false;
	}

}
