<?php
/**
 * Variable Lister
 *
 * In final string format variables should follow format {{user.id}}
 * Supports parameters in the format {{ user.generate_coupon | template: coupon_code, expires: 7  }}
 *
 * @class       AW_Variable_Lister
 * @package     AutomateWoo
 * @since       2.0.0
 */

class AW_Variable_Lister
{
	static $vars = array(
		'order' => array(
			'id',
			'number',
			'status',
			'total',
			'itemscount',
			'items',
			'cross_sells',
			'related_products',
			'billing_phone',
			'view_url',
			'payment_url',
			'meta' => array(
				'key',
			),
		),
		'order_item' => array(
			'attribute' => array(
				'slug'
			)
		),
		'user' => array(
			'id',
			'email',
			'firstname',
			'lastname',
			'username',
			'meta' => array(
				'key',
			),
			'generate_coupon' => array(
				'template',
			),
		),
		'guest' => array(
			'email',
			'generate_coupon' => array(
				'template',
			),
		),
		'comment' => array(
			'id',
			'author_name',
			'author_ip'
		),
		'product' => array(
			'id',
			'title',
			'current_price',
			'regular_price',
			'featured_image',
			'permalink',
			'add_to_cart_url',
			'sku',
			'short_description',
			'meta' => array(
				'key',
			),
		),
		'category' => array(
			'id',
			'title',
			'permalink'
		),
		'wishlist' => array(
			'items',
			'view_link',
			'itemscount'
		),
		'cart' => array(
			'link',
			'items',
			'total'
		),
		'subscription' => array(
			'id',
			'status',
			'payment_method',
			'view_order_url',
			'start_date',
			'next_payment_date',
			'trial_end_date',
			'end_date',
			'last_payment_date',
		),
		'shop' => array(
			'title',
			'tagline',
			'url',
			'admin_email',
			'current_datetime',
			'products' => array(
				'type'
			)
		),

	);


	/**
	 * @return array
	 */
	static function all()
	{
		if ( class_exists( 'WC_Shipment_Tracking' ) )
		{
			self::$vars['order'][] = 'tracking_number';
			self::$vars['order'][] = 'tracking_url';
			self::$vars['order'][] = 'date_shipped';
			self::$vars['order'][] = 'shipping_provider';
		}

		return apply_filters( 'automatewoo_text_variables', self::$vars );
	}


	/**
	 * @param $data_type string e.g. 'user'
	 * @param $value string e.g. 'id
	 * @param $parameters_array
	 *
	 * @return string
	 */
	static function format( $data_type, $value, $parameters_array )
	{
		if ( is_array( $parameters_array ) )
		{
			$after = ': ...';
			$value .= ' | ' . implode( $after . ' , ', $parameters_array ) . $after;
		}
		else
		{
			$value = $parameters_array;
		}

		return '{{ ' . $data_type . '.' . $value . ' }}';
	}

}
